@@ -39,6 +39,7 @@ typedef struct
         int titleColor;                                 // Color of the title, when using config colors
         int artistColor;                                // Artist color, when using config colors
         int enqueuedColor;                              // Color of enqueued files, when using config colors
+        bool mouseEnabled;                              // Accept mouse input or not
         int mouseLeftClickAction;                       // Left mouse action
         int mouseMiddleClickAction;                     // Middle mouse action
         int mouseRightClickAction;                      // Right mouse action
@@ -56,11 +57,16 @@ typedef struct
         bool hideHelp;                                  // No help text at top
         bool allowNotifications;                        // Send desktop notifications or not
         int visualizerHeight;                           // Height in characters of the spectrum visualizer
+        int visualizerColorType;                        // How colors are laid out in the spectrum visualizer
+        bool visualizerBrailleMode;                     // Display the visualizer using braille characteres
         int titleDelay;                                 // Delay when drawing title in track view
         int cacheLibrary;                               // Cache the library or not
         bool quitAfterStopping;                         // Exit kew when the music stops or not
         bool hideGlimmeringText;                        // Glimmering text on the bottom row
         time_t lastTimeAppRan;                          // When did this app run last, used for updating the cached library if it has been modified since that time
+        float tweenFactor;                              // How fast the bars in the visualizer rise (higher value = faster)
+        float tweenFactorFall;                          // How fast the bars in the visualizer fall (higher value = faster)
+        int progressBarType;                  // What method for showing the progress in track view. Default 0.
 } UISettings;
 
 typedef struct
@@ -76,11 +82,12 @@ typedef struct
         bool doNotifyMPRISSwitched;                     // Emit mpris song switched signal
         bool doNotifyMPRISPlaying;                      // Emit mpris music is playing signal
         bool collapseView;                              // Signal that ui needs to collapse the view
+        bool miniMode;
 } UIState;
 
 typedef struct
 {
-        Cache *tempCache;                               // Cache for temporary files
+        Cache *tmpCache;                                // Cache for temporary files
         ViewState currentView;                          // The current view (playlist, library, track) that kew is on
         UIState uiState;
         UISettings uiSettings;
@@ -109,6 +116,7 @@ typedef struct
         char useConfigColors[2];
         char visualizerEnabled[2];
         char visualizerHeight[6];
+        char visualizerColorType[2];
         char titleDelay[6];
         char togglePlaylist[6];
         char toggleBindings[6];
@@ -177,6 +185,7 @@ typedef struct
         char artistColor[2];
         char enqueuedColor[2];
         char titleColor[2];
+        char mouseEnabled[2];
         char mouseLeftClickAction[3];
         char mouseMiddleClickAction[3];
         char mouseRightClickAction[3];
@@ -195,6 +204,13 @@ typedef struct
         char moveSongUp[6];
         char moveSongDown[6];
         char enqueueAndPlay[6];
+        char hardStop[6];
+        char hardAddToRadioFavorites[6];
+        char sortLibrary[6];
+        char visualizerBrailleMode[2];
+        char tweenFactor[12];
+        char tweenFactorFall[12];
+        char progressBarType[2];
 } AppSettings;
 
 #endif
