@@ -14,7 +14,7 @@ enum EventType
         EVENT_QUIT,
         EVENT_TOGGLEREPEAT,
         EVENT_TOGGLEVISUALIZER,
-        EVENT_TOGGLEBLOCKS,
+        EVENT_TOGGLEASCII,
         EVENT_ADDTOMAINPLAYLIST,
         EVENT_DELETEFROMMAINPLAYLIST,
         EVENT_EXPORTPLAYLIST,
@@ -45,7 +45,10 @@ enum EventType
         EVENT_RADIOSEARCH,
         EVENT_MOVESONGUP,
         EVENT_MOVESONGDOWN,
-        EVENT_ENQUEUEANDPLAY
+        EVENT_ENQUEUEANDPLAY,
+        EVENT_ADDTORADIOFAVORITES,
+        EVENT_STOP,
+        EVENT_SORTLIBRARY
 };
 
 struct Event
