@@ -13,17 +13,21 @@ file.c
 */
 
 void getDirectoryFromPath(const char *path, char *directory)
-{  
-    char tempPath[strlen(path) + 1];  // tempPath is needed because the dirname function modifies the input string
-    strcpy(tempPath, path);
+{
+    size_t path_length = strnlen(path, MAXPATHLEN);
+    char tmpPath[path_length + 1];
+    c_strcpy(tmpPath, path, sizeof(tmpPath));
 
-    char *dir = dirname(tempPath);
+    char *dir = dirname(tmpPath);
 
-    strcpy(directory, dir);
+    // Copy directory name to the output buffer
+    snprintf(directory, MAXPATHLEN, "%s", dir);
 
-    if (directory[strlen(directory) - 1] != '/')
+    size_t directory_length = strnlen(directory, MAXPATHLEN);
+    if (directory[directory_length - 1] != '/' && directory_length + 1 < MAXPATHLEN)
     {
-        strcat(directory, "/");
+        // Use snprintf to append '/' at the end of directory
+        snprintf(directory + directory_length, MAXPATHLEN - directory_length, "/");
     }
 }
 
@@ -57,13 +61,13 @@ int isDirectory(const char *path)
 }
 
 // Traverse a directory tree and search for a given file or directory
-int walker(const char *startPath, const char *searching, char *result,
+int walker(const char *startPath, const char *lowCaseSearching, char *result,
            const char *allowedExtensions, enum SearchType searchType, bool exactSearch)
 {
         DIR *d;
         struct dirent *dir;
         struct stat file_stat;
-        char ext[6]; // +1 for null-terminator
+        char ext[100]; // +1 for null-terminator
         regex_t regex;
         int ret = regcomp(&regex, allowedExtensions, REG_EXTENDED);
         if (ret != 0)
@@ -107,7 +111,7 @@ int walker(const char *startPath, const char *searching, char *result,
                 }
 
                 char entryPath[MAXPATHLEN];
-                char *currentDir = getcwd(NULL, 0);                
+                char *currentDir = getcwd(NULL, 0);
                 snprintf(entryPath, sizeof(entryPath), "%s/%s", currentDir, dir->d_name);
                 free(currentDir);
 
@@ -118,23 +122,28 @@ int walker(const char *startPath, const char *searching, char *result,
 
                 if (S_ISDIR(file_stat.st_mode))
                 {
-                        if (((exactSearch && (strcasecmp(dir->d_name, searching) == 0)) || (!exactSearch && c_strcasestr(dir->d_name, searching) != NULL)) &&
+                        char *name = g_utf8_casefold(dir->d_name, -1);
+
+                        if (((exactSearch && (strcasecmp(name, lowCaseSearching) == 0)) || (!exactSearch && c_strcasestr(name, lowCaseSearching, MAXPATHLEN) != NULL)) &&
                             (searchType != FileOnly) && (searchType != SearchPlayList))
                         {
                                 char *curDir = getcwd(NULL, 0);
                                 snprintf(result, MAXPATHLEN, "%s/%s", curDir, dir->d_name);
                                 free(curDir);
+                                free(name);
                                 copyresult = true;
                                 break;
                         }
                         else
                         {
+                                free(name);
+
                                 if (chdir(dir->d_name) == -1)
                                 {
                                         fprintf(stderr, "Failed to change directory: %s\n", dir->d_name);
                                         continue;
                                 }
-                                if (walker(NULL, searching, result, allowedExtensions, searchType, exactSearch) == 0)
+                                if (walker(NULL, lowCaseSearching, result, allowedExtensions, searchType, exactSearch) == 0)
                                 {
                                         copyresult = true;
                                         break;
@@ -154,7 +163,7 @@ int walker(const char *startPath, const char *searching, char *result,
                         }
 
                         char *filename = dir->d_name;
-                        if (strlen(filename) <= 4)
+                        if (strnlen(filename, 256) <= 4)
                         {
                                 continue;
                         }
@@ -165,14 +174,18 @@ int walker(const char *startPath, const char *searching, char *result,
                                 continue;
                         }
 
-                        if ((exactSearch && (strcasecmp(dir->d_name, searching) == 0)) || (!exactSearch && c_strcasestr(dir->d_name, searching) != NULL))
+                        char *name = g_utf8_casefold(dir->d_name, -1);
+
+                        if ((exactSearch && (strcasecmp(name, lowCaseSearching) == 0)) || (!exactSearch && c_strcasestr(name, lowCaseSearching, MAXPATHLEN) != NULL))
                         {
                                 char *curDir = getcwd(NULL, 0);
                                 snprintf(result, MAXPATHLEN, "%s/%s", curDir, dir->d_name);
                                 copyresult = true;
                                 free(curDir);
+                                free(name);
                                 break;
                         }
+                        free(name);
                 }
         }
         closedir(d);
@@ -226,16 +239,16 @@ int expandPath(const char *inputPath, char *expandedPath)
                         }
                 }
 
-                size_t homeDirLen = strlen(homeDir);
-                size_t inputPathLen = strlen(inputPath);
+                size_t homeDirLen = strnlen(homeDir, MAXPATHLEN);
+                size_t inputPathLen = strnlen(inputPath, MAXPATHLEN);
 
                 if (homeDirLen + inputPathLen >= MAXPATHLEN)
                 {
                         return -1; // Expanded path exceeds maximum length
                 }
 
-                strcpy(expandedPath, homeDir);
-                strcat(expandedPath, inputPath);
+                c_strcpy(expandedPath, homeDir, MAXPATHLEN);
+                snprintf(expandedPath + homeDirLen, MAXPATHLEN - homeDirLen, "%s", inputPath);
         }
         else // Handle if path is not prefixed with '~'
         {
@@ -267,49 +280,6 @@ int createDirectory(const char *path)
         return -1; // Failed to create directory
 }
 
-int removeDirectory(const char *path)
-{
-        struct stat st;
-
-        // Check if path exists
-        if (stat(path, &st) != 0)
-                return -1; // Path does not exist
-
-        // Check if it is a directory
-        if (!S_ISDIR(st.st_mode))
-                return -1; // Path exists but is not a directory
-
-        DIR *dir = opendir(path);
-
-        if (dir == NULL)
-                return -1; // Failed to open directory
-
-        struct dirent *entry;
-        char filePath[MAXPATHLEN];
-
-        // Remove all entries in the directory
-        while ((entry = readdir(dir)) != NULL)
-        {
-                if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
-                        continue; // Skip current and parent directories
-
-                snprintf(filePath, sizeof(filePath), "%s/%s", path, entry->d_name);
-
-                if (entry->d_type == DT_DIR)
-                        removeDirectory(filePath); // Recursively remove subdirectories
-                else
-                        remove(filePath); // Remove regular file
-        }
-
-        closedir(dir);
-
-        // Remove the directory itself
-        if (rmdir(path) == 0)
-                return 0; // Directory removed successfully
-
-        return -1; // Failed to remove directory
-}
-
 int deleteFile(const char *filePath)
 {
         if (remove(filePath) == 0)
@@ -324,77 +294,42 @@ int deleteFile(const char *filePath)
 
 int isInTempDir(const char *path)
 {
-        const char *tempDir = getenv("TMPDIR");
-        if (tempDir == NULL)
+        const char *tmpDir = getenv("TMPDIR");
+        if (tmpDir == NULL || strnlen(tmpDir, PATH_MAX) >= PATH_MAX)
         {
-#ifdef __APPLE__
-                tempDir = "/tmp";
-#else
-                tempDir = "/tmp";
-#endif
+                tmpDir = "/tmp";
         }
 
-        return (startsWith(path, tempDir));
-}
-
-void deleteTempDir()
-{
-        const char *tempDir = getenv("TMPDIR");
-        if (tempDir == NULL)
-        {
-#ifdef __APPLE__
-                tempDir = "/tmp";
-#else
-                tempDir = "/tmp";
-#endif
-        }
-        else
-        {
-        }
-        char dirPath[MAXPATHLEN];
-        struct passwd *pw = getpwuid(getuid());
-        const char *username = pw->pw_name;
-        snprintf(dirPath, MAXPATHLEN, "%s/kew/%s", tempDir, username);
-        removeDirectory(dirPath);
-}
-
-bool checkFileBelowMaxSize(const char *filePath, int maxSize)
-{
-        struct stat st;
-
-        if (stat(filePath, &st) == 0)
-        {
-                return (st.st_size <= maxSize);
-        }
-
-        perror("stat failed");
-
-        return false;
+        return (pathStartsWith(path, tmpDir));
 }
 
 void generateTempFilePath(char *filePath, const char *prefix, const char *suffix)
 {
-        const char *tempDir = getenv("TMPDIR");
-        if (tempDir == NULL)
+        const char *tmpDir = getenv("TMPDIR");
+        if (tmpDir == NULL || strnlen(tmpDir, PATH_MAX) >= PATH_MAX)
         {
-                tempDir = "/tmp";
+                tmpDir = "/tmp";
         }
 
-        char dirPath[MAXPATHLEN];
         struct passwd *pw = getpwuid(getuid());
-        const char *username = pw->pw_name;
-        snprintf(dirPath, MAXPATHLEN, "%s/kew", tempDir);
+        const char *username = pw ? pw->pw_name : "unknown";
+
+        char dirPath[MAXPATHLEN];
+        snprintf(dirPath, sizeof(dirPath), "%s/kew", tmpDir);
         createDirectory(dirPath);
-        snprintf(dirPath, MAXPATHLEN, "%s/kew/%s", tempDir, username);
+        snprintf(dirPath, sizeof(dirPath), "%s/kew/%s", tmpDir, username);
         createDirectory(dirPath);
 
         char randomString[7];
-
         for (int i = 0; i < 6; ++i)
         {
                 randomString[i] = 'a' + rand() % 26;
         }
         randomString[6] = '\0';
 
-        snprintf(filePath, MAXPATHLEN + 7, "%s/%s%.6s%s", dirPath, prefix, randomString, suffix);
+        int written = snprintf(filePath, MAXPATHLEN, "%s/%s%.6s%s", dirPath, prefix, randomString, suffix);
+        if (written < 0 || written >= MAXPATHLEN)
+        {
+                filePath[0] = '\0';
+        }
 }
