@@ -23,7 +23,7 @@
 #endif
 
 #ifndef AUDIO_EXTENSIONS
-#define AUDIO_EXTENSIONS "\\.(m4a|aac|mp3|ogg|flac|wav|opus)$"
+#define AUDIO_EXTENSIONS "(m4a|aac|mp3|ogg|flac|wav|opus)$"
 #endif
 
 enum SearchType
@@ -47,14 +47,10 @@ int expandPath(const char *inputPath, char *expandedPath);
 
 int createDirectory(const char *path);
 
-int removeDirectory(const char *path);
-
 int deleteFile(const char *filePath);
 
 void generateTempFilePath(char *filePath, const char *prefix, const char *suffix);
 
-void deleteTempDir(void);
-
 int isInTempDir(const char *path);
 
 int existsFile(const char *fname);
