@@ -76,7 +76,6 @@ FILE *logFile = NULL;
 struct winsize windowSize;
 char digitsPressed[MAX_SEQ_LEN];
 int digitsPressedCount = 0;
-static unsigned int updateCounter = 0;
 bool startFromTop = false;
 int lastNotifiedId = -1;
 bool songWasRemoved = false;
@@ -85,9 +84,10 @@ GMainLoop *main_loop;
 EventMapping keyMappings[NUM_KEY_MAPPINGS];
 struct timespec lastInputTime;
 bool exactSearch = false;
-int fuzzySearchThreshold = 2;
+int fuzzySearchThreshold = 4;
 int maxDigitsPressedCount = 9;
 int isNewSearchTerm = false;
+bool wasEndOfList = false;
 
 void updateLastInputTime(void)
 {
@@ -157,7 +157,7 @@ struct Event processInput()
                     strcmp(seq + 1, settings.hardNextPage) == 0 || strcmp(seq + 1, settings.hardPrevPage) == 0)
                 {
                         keyReleased = 0;
-                        readInputSequence(tmpSeq, sizeof(tmpSeq)); // dummy read to prevent scrolling after key released
+                        readInputSequence(tmpSeq, sizeof(tmpSeq)); // Dummy read to prevent scrolling after key released
                         break;
                 }
 
@@ -204,7 +204,12 @@ struct Event processInput()
                         isNewSearchTerm = true;
                         event.type = EVENT_RADIOSEARCH;
                 }
-                else if (((strnlen(event.key, sizeof(event.key)) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\t' && event.key[0] != '\r') || strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0) && strcmp(event.key, "Z") != 0 && strcmp(event.key, "X") != 0 && strcmp(event.key, "C") != 0 && strcmp(event.key, "V") != 0 && strcmp(event.key, "B") != 0 && strcmp(event.key, "N") != 0)
+                else if (((strnlen(event.key, sizeof(event.key)) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\t' && event.key[0] != '\r') ||
+                          strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0) &&
+                         strcmp(event.key, "Z") != 0 && strcmp(event.key, "X") != 0 &&
+                         strcmp(event.key, "C") != 0 && strcmp(event.key, "V") != 0 &&
+                         strcmp(event.key, "F") != 0 && strcmp(event.key, "S") != 0 &&
+                         strcmp(event.key, "B") != 0 && strcmp(event.key, "N") != 0)
                 {
                         addToRadioSearchText(event.key);
                         resetRadioSearchResult();
@@ -213,7 +218,7 @@ struct Event processInput()
                 }
                 else if (event.key[0] == '\n')
                 {
-                        if (isNewSearchTerm)
+                        if (isNewSearchTerm && hasRadioSearchText())
                         {
                                 radioSearch();
                                 event.type = EVENT_RADIOSEARCH;
@@ -231,7 +236,7 @@ struct Event processInput()
         for (int i = 0; i < NUM_KEY_MAPPINGS; i++)
         {
                 if (keyMappings[i].seq[0] != '\0' &&
-                    ((seq[0] == '\033' && strcmp(seq, "\033\n") != 0 && strnlen(seq, MAX_SEQ_LEN) > 1 && strcmp(seq + 1, keyMappings[i].seq) == 0) ||
+                    ((seq[0] == '\033' && strnlen(seq, MAX_SEQ_LEN) > 1 && strcmp(seq, "\033\n") != 0 && strcmp(seq + 1, keyMappings[i].seq) == 0) ||
                      strcmp(seq, keyMappings[i].seq) == 0))
                 {
                         if (event.type == EVENT_SEARCH && keyMappings[i].eventType != EVENT_GOTOSONG)
@@ -248,16 +253,25 @@ struct Event processInput()
                         break;
                 }
 
+                // Received mouse input instead of keyboard input
+                if (keyMappings[i].seq[0] != '\0' && strnlen(seq, MAX_SEQ_LEN) > 3 && strncmp(seq, "\033[M", 3) == 0 &&
+                    ((strncmp(seq + 1, keyMappings[i].seq, 3) == 0) ||
+                     strncmp(seq, keyMappings[i].seq, 3) == 0))
+                {
+                        event.type = keyMappings[i].eventType;
+                        break;
+                }
+        }
+
+        for (int i = 0; i < NUM_KEY_MAPPINGS; i++)
+        {
                 if (strcmp(seq, "\033\n") == 0 && strcmp(keyMappings[i].seq, "^M") == 0) // ALT+ENTER
                 {
                         event.type = keyMappings[i].eventType;
                         break;
                 }
 
-                // Received mouse input instead of keyboard input
-                if (keyMappings[i].seq[0] != '\0' && strnlen(seq, MAX_SEQ_LEN) > 3 && strncmp(seq, "\033[M", 3) == 0 &&
-                    ((strncmp(seq + 1, keyMappings[i].seq, 3) == 0) ||
-                     strncmp(seq, keyMappings[i].seq, 3) == 0))
+                if (strcmp(seq, keyMappings[i].seq) == 0 && strnlen(seq, MAX_SEQ_LEN) > 1) // ALT+something
                 {
                         event.type = keyMappings[i].eventType;
                         break;
@@ -331,7 +345,6 @@ struct Event processInput()
 
 void setEndOfListReached(AppState *state)
 {
-        state->currentView = LIBRARY_VIEW;
         loadedNextSong = false;
         audioData.endOfListReached = true;
         usingSongDataA = false;
@@ -339,13 +352,21 @@ void setEndOfListReached(AppState *state)
         audioData.currentFileIndex = 0;
         audioData.restart = true;
         loadingdata.loadA = true;
-        emitMetadataChanged("", "", "", "", "/org/mpris/MediaPlayer2/TrackList/NoTrack", NULL, 0);
-        emitPlaybackStoppedMpris();
+
         pthread_mutex_lock(&dataSourceMutex);
         cleanupPlaybackDevice();
         pthread_mutex_unlock(&dataSourceMutex);
         stopped = true;
         refresh = true;
+
+        if (isRepeatListEnabled())
+                repeatList();
+        else
+        {
+                emitPlaybackStoppedMpris();
+                emitMetadataChanged("", "", "", "", "/org/mpris/MediaPlayer2/TrackList/NoTrack", NULL, 0);
+                state->currentView = LIBRARY_VIEW;
+        }
 }
 
 void notifyMPRISSwitch(SongData *currentSongData)
@@ -355,7 +376,7 @@ void notifyMPRISSwitch(SongData *currentSongData)
 
         gint64 length = getLengthInMicroSec(currentSongData->duration);
 
-        // update mpris
+        // Update mpris
         emitMetadataChanged(
             currentSongData->metadata->title,
             currentSongData->metadata->artist,
@@ -524,7 +545,7 @@ FileSystemEntry *enqueue(AppState *state, FileSystemEntry *entry)
 
         pthread_mutex_lock(&(playlist.mutex));
 
-        firstEnqueuedEntry = enqueueSongs(entry, &state->uiState);
+        firstEnqueuedEntry = enqueueSongs(entry, &(state->uiState));
         resetListAfterDequeuingPlayingSong(state);
 
         pthread_mutex_unlock(&(playlist.mutex));
@@ -532,53 +553,40 @@ FileSystemEntry *enqueue(AppState *state, FileSystemEntry *entry)
         return firstEnqueuedEntry;
 }
 
-void enqueueAndPlay(AppState *state)
+void playPreProcessing()
 {
-        if (state->currentView == LIBRARY_VIEW || state->currentView == SEARCH_VIEW)
+        wasEndOfList = false;
+        if (audioData.endOfListReached)
+                wasEndOfList = true;
+}
+
+void playPostProcessing()
+{
+        if ((songWasRemoved && currentSong != NULL))
         {
-                if (isRadioPlaying())
+                songWasRemoved = false;
+        }
+
+        if (wasEndOfList)
+        {
+                skipOutOfOrder = false;
+        }
+
+        audioData.endOfListReached = false;
+}
+
+void handleAddToRadioFavorites(AppState *state)
+{
+        if (state->currentView == RADIOSEARCH_VIEW)
+        {
+                RadioSearchResult *station = getCurrentRadioSearchEntry();
+
+                if (station)
                 {
-                        stopRadio();
-                        setEOFReached();
+                        addToRadioFavorites(station);
+                        refresh = true;
                 }
         }
-
-        FileSystemEntry *firstEnqueuedEntry = NULL;
-
-        bool wasEmpty = (playlist.count == 0);
-
-        if (state->currentView == LIBRARY_VIEW)
-        {
-                firstEnqueuedEntry = enqueue(state, getCurrentLibEntry());
-        }
-
-        if (state->currentView == SEARCH_VIEW)
-        {
-                FileSystemEntry *entry = getCurrentSearchEntry();
-                firstEnqueuedEntry = enqueue(state, entry);
-
-                setChosenDir(entry);
-        }
-
-        if (firstEnqueuedEntry && !wasEmpty)
-        {
-                Node *song = findPathInPlaylist(firstEnqueuedEntry->fullPath, &playlist);
-
-                loadedNextSong = true;
-
-                nextSongNeedsRebuilding = false;
-
-                unloadSongA(state);
-                unloadSongB(state);
-
-                usingSongDataA = false;
-                audioData.currentFileIndex = 0;
-                loadingdata.loadA = true;
-
-                playbackPlay(&totalPauseSeconds, &pauseSeconds);
-
-                play(song);
-        }
 }
 
 void handleGoToSong(AppState *state)
@@ -597,7 +605,7 @@ void handleGoToSong(AppState *state)
 
                 setChosenDir(entry);
 
-                enqueueSongs(entry, &state->uiState);
+                enqueueSongs(entry, &(state->uiState));
 
                 resetListAfterDequeuingPlayingSong(state);
 
@@ -622,6 +630,10 @@ void handleGoToSong(AppState *state)
                                         stopRadio();
                                         setEOFReached();
                                 }
+                                else
+                                {
+                                        cleanupPlaybackDevice();
+                                }
 
                                 loadedNextSong = true;
 
@@ -634,25 +646,18 @@ void handleGoToSong(AppState *state)
                                 audioData.currentFileIndex = 0;
                                 loadingdata.loadA = true;
 
-                                bool wasEndOfList = false;
-                                if (audioData.endOfListReached)
-                                        wasEndOfList = true;
+                                playPreProcessing();
 
-                                skipToSong(state->uiState.chosenNodeId, true);
+                                playbackPlay(&totalPauseSeconds, &pauseSeconds);
 
-                                if ((songWasRemoved && currentSong != NULL))
-                                {
-                                        usingSongDataA = !usingSongDataA;
-                                        songWasRemoved = false;
-                                }
+                                Node *found = NULL;
+                                findNodeInList(&playlist, state->uiState.chosenNodeId, &found);
+                                play(found);
 
-                                if (wasEndOfList)
-                                {
-                                        usingSongDataA = true;
-                                        skipOutOfOrder = false;
-                                }
+                                playPostProcessing();
 
-                                audioData.endOfListReached = false;
+                                skipOutOfOrder = false;
+                                usingSongDataA = true;
                         }
                 }
                 else
@@ -676,6 +681,70 @@ void handleGoToSong(AppState *state)
         }
 }
 
+void enqueueAndPlay(AppState *state)
+{
+        if (state->currentView == LIBRARY_VIEW || state->currentView == SEARCH_VIEW)
+        {
+                if (isRadioPlaying())
+                {
+                        stopRadio();
+                        setEOFReached();
+                }
+        }
+
+        FileSystemEntry *firstEnqueuedEntry = NULL;
+
+        bool wasEmpty = (playlist.count == 0);
+
+        playPreProcessing();
+
+        if (state->currentView == PLAYLIST_VIEW)
+        {
+                handleGoToSong(state);
+                return;
+        }
+
+        if (state->currentView == LIBRARY_VIEW)
+        {
+                firstEnqueuedEntry = enqueue(state, getCurrentLibEntry());
+        }
+
+        if (state->currentView == SEARCH_VIEW)
+        {
+                FileSystemEntry *entry = getCurrentSearchEntry();
+                firstEnqueuedEntry = enqueue(state, entry);
+
+                setChosenDir(entry);
+        }
+
+        if (firstEnqueuedEntry && !wasEmpty)
+        {
+                Node *song = findPathInPlaylist(firstEnqueuedEntry->fullPath, &playlist);
+
+                loadedNextSong = true;
+
+                nextSongNeedsRebuilding = false;
+
+                cleanupPlaybackDevice();
+
+                unloadSongA(state);
+                unloadSongB(state);
+
+                usingSongDataA = false;
+                audioData.currentFileIndex = 0;
+                loadingdata.loadA = true;
+
+                playbackPlay(&totalPauseSeconds, &pauseSeconds);
+
+                play(song);
+
+                playPostProcessing();
+
+                skipOutOfOrder = false;
+                usingSongDataA = true;
+        }
+}
+
 void gotoBeginningOfPlaylist(AppState *state)
 {
         digitsPressed[0] = 1;
@@ -720,8 +789,8 @@ void handleInput(AppState *state)
         case EVENT_TOGGLEREPEAT:
                 toggleRepeat();
                 break;
-        case EVENT_TOGGLEBLOCKS:
-                toggleBlocks(&settings, &(state->uiSettings));
+        case EVENT_TOGGLEASCII:
+                toggleAscii(&settings, &(state->uiSettings));
                 break;
         case EVENT_SHUFFLE:
                 toggleShuffle();
@@ -820,6 +889,16 @@ void handleInput(AppState *state)
         case EVENT_ENQUEUEANDPLAY:
                 enqueueAndPlay(state);
                 break;
+        case EVENT_ADDTORADIOFAVORITES:
+                handleAddToRadioFavorites(state);
+                break;
+        case EVENT_STOP:
+                stop();
+                break;
+        case EVENT_SORTLIBRARY:
+                sortLibrary();
+                break;
+
         default:
                 fastForwarding = false;
                 rewinding = false;
@@ -846,7 +925,7 @@ void updatePlayer(UIState *uis)
         struct winsize ws;
         ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws);
 
-        // check if window has changed size
+        // Check if window has changed size
         if (ws.ws_col != windowSize.ws_col || ws.ws_row != windowSize.ws_row)
         {
                 uis->resizeFlag = 1;
@@ -948,7 +1027,7 @@ void loadAudioData(AppState *state)
         else if (currentSong != NULL && (nextSongNeedsRebuilding || nextSong == NULL) && !songLoading)
         {
                 loadNextSong();
-                determineSongAndNotify(&state->uiSettings);
+                determineSongAndNotify(&(state->uiSettings));
         }
 }
 
@@ -987,12 +1066,6 @@ void handleSkipOutOfOrder(void)
         }
 }
 
-void resetClock(void)
-{
-        resetTimeCount();
-        clock_gettime(CLOCK_MONOTONIC, &start_time);
-}
-
 void prepareNextSong(AppState *state)
 {
         resetClock();
@@ -1037,7 +1110,7 @@ void handleSkipFromStopped()
 
 void updatePlayerStatus(AppState *state)
 {
-        updatePlayer(&state->uiState);
+        updatePlayer(&(state->uiState));
 
         reconnectRadioIfNeeded();
 
@@ -1082,8 +1155,8 @@ gboolean mainloop_callback(gpointer data)
 
         updateCounter++;
 
-        // Update every other time or if searching (search needs to update often to detect keypresses)
-        if (updateCounter % 2 == 0 || appState.currentView == SEARCH_VIEW || appState.currentView == RADIOSEARCH_VIEW)
+        // Different views run at different speeds to lower the impact on system requirements
+        if ((updateCounter % 4 == 0 && (appState.currentView == SEARCH_VIEW || appState.currentView == RADIOSEARCH_VIEW)) || (appState.currentView == TRACK_VIEW || appState.uiState.miniMode) || updateCounter % 6 == 0)
         {
                 processDBusEvents();
 
@@ -1155,13 +1228,15 @@ void initFirstPlay(Node *song, AppState *state)
         else
                 emitPlaybackStoppedMpris();
 
-        g_timeout_add(50, mainloop_callback, NULL);
+        g_timeout_add(17, mainloop_callback, NULL);
         g_main_loop_run(main_loop);
         g_main_loop_unref(main_loop);
 }
 
 void cleanupOnExit()
 {
+        stopRadio();
+
         pthread_mutex_lock(&dataSourceMutex);
 
         resetAllDecoders();
@@ -1172,8 +1247,6 @@ void cleanupOnExit()
                 cleanupAudioContext();
         }
 
-        stopRadio();
-
         emitPlaybackStoppedMpris();
 
         bool noMusicFound = false;
@@ -1188,12 +1261,12 @@ void cleanupOnExit()
         if (!userData.songdataADeleted)
         {
                 userData.songdataADeleted = true;
-                unloadSongData(&loadingdata.songdataA, &appState);
+                unloadSongData(&(loadingdata.songdataA), &appState);
         }
         if (!userData.songdataBDeleted)
         {
                 userData.songdataBDeleted = true;
-                unloadSongData(&loadingdata.songdataB, &appState);
+                unloadSongData(&(loadingdata.songdataB), &appState);
         }
 
         freeSearchResults();
@@ -1205,9 +1278,9 @@ void cleanupOnExit()
         enableInputBuffering();
         setConfig(&settings, &(appState.uiSettings));
         saveSpecialPlaylist(settings.path);
-        freeAudioBuffer();
-        deleteCache(appState.tempCache);
+        deleteCache(appState.tmpCache);
         freeMainDirectoryTree(&appState);
+        freeAndwriteRadioFavorites();
         deletePlaylist(&playlist);
         deletePlaylist(originalPlaylist);
         deletePlaylist(specialPlaylist);
@@ -1219,6 +1292,8 @@ void cleanupOnExit()
         pthread_mutex_destroy(&(switchMutex));
         pthread_mutex_unlock(&dataSourceMutex);
         pthread_mutex_destroy(&(dataSourceMutex));
+        destroyRadioMutexes();
+        freeVisuals();
         freeLastCover();
 #ifdef USE_DBUS
         cleanupDbusConnection();
@@ -1235,7 +1310,8 @@ void cleanupOnExit()
         printf("\n");
         showCursor();
         exitAlternateScreenBuffer();
-        disableMouseButtons();
+        if (appState.uiSettings.mouseEnabled)
+                disableTerminalMouseButtons();
 
         if (noMusicFound)
         {
@@ -1291,7 +1367,7 @@ void initResize()
 
         struct sigaction sa;
         sa.sa_handler = resetResizeFlag;
-        sigemptyset(&sa.sa_mask);
+        sigemptyset(&(sa.sa_mask));
         sa.sa_flags = 0;
         sigaction(SIGALRM, &sa, NULL);
 }
@@ -1303,7 +1379,7 @@ void init(AppState *state)
         ioctl(STDOUT_FILENO, TIOCGWINSZ, &windowSize);
         enableScrolling();
         setNonblockingMode();
-        state->tempCache = createCache();
+        state->tmpCache = createCache();
         c_strcpy(loadingdata.filePath, "", sizeof(loadingdata.filePath));
         loadingdata.songdataA = NULL;
         loadingdata.songdataB = NULL;
@@ -1312,15 +1388,17 @@ void init(AppState *state)
         audioData.restart = true;
         userData.songdataADeleted = true;
         userData.songdataBDeleted = true;
-        initAudioBuffer();
         unsigned int seed = (unsigned int)time(NULL);
         srand(seed);
         pthread_mutex_init(&dataSourceMutex, NULL);
         pthread_mutex_init(&switchMutex, NULL);
         pthread_mutex_init(&(loadingdata.mutex), NULL);
         pthread_mutex_init(&(playlist.mutex), NULL);
+        initRadioMutexes();
         createLibrary(&settings, state);
+        createRadioFavorites();
         curl_global_init(CURL_GLOBAL_DEFAULT);
+        setlocale(LC_CTYPE, "");
         fflush(stdout);
 
 #ifdef DEBUG
@@ -1511,7 +1589,7 @@ void exitIfAlreadyRunning()
                         fclose(pidfile);
                         if (isProcessRunning(pid))
                         {
-                                fprintf(stderr, "An instance of kew is already running. Pid: %d.\n", pid);
+                                fprintf(stderr, "An instance of kew is already running. Pid: %d. Type 'kill %d' to remove it.\n", pid, pid);
                                 exit(1);
                         }
                         else
@@ -1648,12 +1726,22 @@ void setMusicPath()
                 exit(1);
 }
 
+void enableMouse(UISettings *ui)
+{
+        if (ui->mouseEnabled)
+        {
+                enableTerminalMouseButtons();
+        }
+}
+
 void initState(AppState *state)
 {
         state->uiSettings.uiEnabled = true;
         state->uiSettings.color.r = 125;
         state->uiSettings.color.g = 125;
         state->uiSettings.color.b = 125;
+        state->uiSettings.tweenFactor = 0.23f;
+        state->uiSettings.tweenFactorFall = 0.13f;
         state->uiSettings.visualizerEnabled = true;
         state->uiSettings.coverEnabled = true;
         state->uiSettings.hideLogo = false;
@@ -1662,9 +1750,20 @@ void initState(AppState *state)
         state->uiSettings.hideGlimmeringText = false;
         state->uiSettings.coverAnsi = false;
         state->uiSettings.visualizerHeight = 5;
+        state->uiSettings.visualizerColorType = 0;
         state->uiSettings.titleDelay = 9;
+        state->uiSettings.visualizerBrailleMode = false;
         state->uiSettings.cacheLibrary = -1;
         state->uiSettings.useConfigColors = false;
+        state->uiSettings.mouseEnabled = true;
+        state->uiSettings.mouseLeftClickAction = 0;
+        state->uiSettings.mouseMiddleClickAction = 1;
+        state->uiSettings.mouseRightClickAction = 2;
+        state->uiSettings.mouseScrollUpAction = 3;
+        state->uiSettings.mouseScrollDownAction = 4;
+        state->uiSettings.mouseAltScrollUpAction = 7;
+        state->uiSettings.mouseAltScrollDownAction = 8;
+        state->uiSettings.progressBarType = 0;
         state->uiState.numDirectoryTreeEntries = 0;
         state->uiState.numProgressBars = 35;
         state->uiState.chosenNodeId = 0;
@@ -1676,7 +1775,7 @@ void initState(AppState *state)
         state->uiState.doNotifyMPRISSwitched = false;
         state->uiState.doNotifyMPRISPlaying = false;
         state->uiState.collapseView = false;
-        state->tempCache = NULL;
+        state->tmpCache = NULL;
 
         radioContext.buf.stale = false;
 }
@@ -1684,8 +1783,9 @@ void initState(AppState *state)
 void initializeStateAndSettings(AppState *appState, AppSettings *settings)
 {
         initState(appState);
-        getConfig(settings, &appState->uiSettings);
-        mapSettingsToKeys(settings, &appState->uiSettings, keyMappings);
+        getConfig(settings, &(appState->uiSettings));
+        mapSettingsToKeys(settings, &(appState->uiSettings), keyMappings);
+        enableMouse(&(appState->uiSettings));
 }
 
 int main(int argc, char *argv[])
@@ -1705,8 +1805,6 @@ int main(int argc, char *argv[])
                 exit(0);
         }
 
-        enableMouseButtons();
-
         initializeStateAndSettings(&appState, &settings);
 
         if (argc == 3 && (strcmp(argv[1], "path") == 0))
