@@ -23,19 +23,20 @@ extern "C"
 #include <stdint.h>
 #include <string.h>
 #include <stdint.h>
+#include "common.h"
 
         typedef enum
         {
                 k_unknown = 0,
                 k_aac = 1,
-                k_rawAAC = 2, // raw aac (.aac file) decoding is included here for convenience although they are not .m4a files
+                k_rawAAC = 2, // Raw aac (.aac file) decoding is included here for convenience although they are not .m4a files
                 k_ALAC = 3,
                 k_FLAC = 4
         } k_m4adec_filetype;
 
         typedef struct
         {
-                ma_data_source_base ds; /* The m4a decoder can be used independently as a data source. */
+                ma_data_source_base ds; // The m4a decoder can be used independently as a data source.
                 ma_read_proc onRead;
                 ma_seek_proc onSeek;
                 ma_tell_proc onTell;
@@ -52,6 +53,7 @@ extern "C"
                 int bitDepth;
                 ma_uint32 sampleRate;
                 ma_uint32 channels;
+                ma_uint32 avgBitRate;
                 double duration;
                 unsigned long totalFrames;
 
@@ -421,7 +423,7 @@ extern "C"
                                 if (atom_size_out)
                                         *atom_size_out = atom_size - 8;
 
-                                return 1; // found
+                                return 1; // Found
                         }
 
                         if (fseek(fp, atom_size - 8, SEEK_CUR) != 0)
@@ -478,25 +480,25 @@ extern "C"
                 if (!find_atom(fp, FOUR_CHAR_INT('s', 't', 's', 'd'), atom_size, &atom_size))
                         return 0;
 
-                fseek(fp, 8, SEEK_CUR); // skip stsd header (version+entry)
+                fseek(fp, 8, SEEK_CUR); // Skip stsd header (version+entry)
 
                 read_u32be(fp); // uint32_t sample_entry_size
                 uint32_t sample_entry_fourcc = read_u32be(fp);
                 if (sample_entry_fourcc != FOUR_CHAR_INT('a', 'l', 'a', 'c'))
                         return 0;
 
-                fseek(fp, 28, SEEK_CUR); // skip audio sample entry fields
+                fseek(fp, 28, SEEK_CUR); // Skip audio sample entry fields
 
                 uint32_t config_atom_size = read_u32be(fp);
                 uint32_t config_atom_fourcc = read_u32be(fp);
                 if (config_atom_fourcc != FOUR_CHAR_INT('a', 'l', 'a', 'c'))
                         return 0;
 
-                fseek(fp, 4, SEEK_CUR); // skip 1-byte version and 3-byte flags (4 bytes total)!
+                fseek(fp, 4, SEEK_CUR); // Skip 1-byte version and 3-byte flags (4 bytes total)!
 
                 uint32_t alac_dsi_size = config_atom_size - 12; // size(4)+fourcc(4)+version/flags(4) total=12 bytes overhead
                 if (alac_dsi_size < 24 || alac_dsi_size > 64)
-                        return 0; // sanity check
+                        return 0; // Sanity check
 
                 if (fread(dsi_out, 1, alac_dsi_size, fp) != alac_dsi_size)
                         return 0;
@@ -627,7 +629,7 @@ extern "C"
                         unsigned char objectType = decoder_config[0];
                         if (objectType == 5 || objectType >= 29)
                         {
-                                printf("File is encoded with HE-AAC which is not supported\n");
+                                setErrorMessage("File is encoded with HE-AAC which is not supported");
                                 free(frameData);
                                 free(decoder_config);
                                 fclose(fp);
@@ -727,7 +729,7 @@ extern "C"
 
                         pM4a->current_sample = 0;
                         pM4a->total_samples = pM4a->track->sample_count;
-
+                        pM4a->avgBitRate = pM4a->track->avg_bitrate_bps;
                         uint8_t alac_dsi[32];
                         size_t alac_dsi_size;
 
@@ -746,6 +748,7 @@ extern "C"
                                 pM4a->bitDepth = alacConfig.bitDepth;
                                 pM4a->sampleRate = (ma_uint32)alacConfig.sampleRate;
                                 pM4a->channels = (ma_uint32)alacConfig.numChannels;
+                                pM4a->avgBitRate = (ma_uint32)alacConfig.avgBitRate;
 
                                 if (pM4a->bitDepth == 32)
                                 {
@@ -972,7 +975,7 @@ extern "C"
                                 // Remove support for HE-AAC components (SBR or PS)
                                 if (pM4a->frameInfo.sbr || pM4a->frameInfo.ps)
                                 {
-                                        printf("File is encoded with HE-AAC which is not supported\n");
+                                        // File is encoded with HE-AAC which is not supported
                                         return MA_ERROR;
                                 }
 
