@@ -62,6 +62,12 @@ FileSystemEntry *lastEntry = NULL;
 FileSystemEntry *chosenDir = NULL;
 FileSystemEntry *library = NULL;
 
+const char *LOGO[] = {
+    " __\n",
+    "|  |--.-----.--.--.--.\n",
+    "|    <|  -__|  |  |  |\n",
+    "|__|__|_____|________|"};
+
 int calcIdealImgSize(int *width, int *height, const int visualizerHeight, const int metatagHeight)
 {
         float aspectRatio = calcAspectRatio();
@@ -128,7 +134,8 @@ void printHelp()
                " Enter to select or replay a song.\n"
                " Switch tracks with ←, → or h, l keys.\n"
                " Volume is adjusted with + (or =) and -.\n"
-               " Space, p or right mouse to toggle pause.\n"
+               " Space, p or right mouse to play or pause.\n"
+               " Shift+s to stop.\n"
                " F2 to show/hide playlist view.\n"
                " F3 to show/hide library view.\n"
                " F4 to show/hide track view.\n"
@@ -168,14 +175,11 @@ int printLogo(SongData *songData, UISettings *ui)
 
         if (!ui->hideLogo)
         {
-                printBlankSpaces(indent);
-                printf(" __\n");
-                printBlankSpaces(indent);
-                printf("|  |--.-----.--.--.--.\n");
-                printBlankSpaces(indent);
-                printf("|    <|  -__|  |  |  |\n");
-                printBlankSpaces(indent);
-                printf("|__|__|_____|________|");
+                for (size_t i = 0; i < sizeof(LOGO) / sizeof(LOGO[0]); i++)
+                {
+                        printBlankSpaces(indent);
+                        printf("%s", LOGO[i]);
+                }
 
                 logoWidth = 22;
                 height += 3;
@@ -193,17 +197,26 @@ int printLogo(SongData *songData, UISettings *ui)
 
                 char title[MAXPATHLEN] = {0};
 
+                int titleLength = strnlen(songData->metadata->title, METADATA_MAX_SIZE);
+                char prettyTitle[titleLength + 1];
+
+                strncpy(prettyTitle, songData->metadata->title, titleLength);
+                prettyTitle[titleLength] = '\0';
+
+                removeUnneededChars(prettyTitle, titleLength);
+                trim(prettyTitle, titleLength);
+
                 if (ui->hideLogo && songData->metadata->artist[0] != '\0')
                 {
                         printBlankSpaces(indent);
                         snprintf(title, MAXPATHLEN, "%s - %s",
-                                 songData->metadata->artist, songData->metadata->title);
+                                 songData->metadata->artist, prettyTitle);
                 }
                 else
                 {
                         if (ui->hideLogo)
                                 printBlankSpaces(indent);
-                        c_strcpy(title, songData->metadata->title, METADATA_MAX_SIZE - 1);
+                        c_strcpy(title, prettyTitle, METADATA_MAX_SIZE - 1);
                         title[MAXPATHLEN - 1] = '\0';
                 }
 
@@ -267,7 +280,7 @@ void printCover(SongData *songdata, UISettings *ui)
 {
         clearScreen();
 
-        if (songdata->cover != NULL && ui->coverEnabled)
+        if (songdata != NULL && songdata->cover != NULL && ui->coverEnabled)
         {
                 if (!ui->coverAnsi)
                 {
@@ -291,9 +304,12 @@ void printCover(SongData *songdata, UISettings *ui)
 
 void printTitleWithDelay(const char *text, int delay, int maxWidth)
 {
-        int length = strnlen(text, maxWidth);
-        int max = (maxWidth > length) ? length : maxWidth;
-        for (int i = 0; i <= max; i++)
+        int max = strnlen(text, maxWidth);
+
+        if (max == maxWidth) // For long names
+                max -= 2;    // Accommodate for the cursor that we display after the name.
+
+        for (int i = 0; i <= max && delay; i++)
         {
                 printf("\r ");
                 printBlankSpaces(indent);
@@ -304,8 +320,7 @@ void printTitleWithDelay(const char *text, int delay, int maxWidth)
                 printf("█");
                 fflush(stdout);
 
-                if (delay)
-                        c_sleep(delay);
+                c_sleep(delay);
         }
         if (delay)
                 c_sleep(delay * 20);
@@ -378,7 +393,17 @@ void printBasicMetadata(TagSettings const *metadata, UISettings *ui)
                         printf("\033[1;38;2;%03u;%03u;%03um", pixel.r, pixel.g, pixel.b);
                 }
 
-                printTitleWithDelay(metadata->title, ui->titleDelay, maxWidth - 2);
+                // Clean up title before printing
+                int titleLength = strnlen(metadata->title, maxWidth);
+                char prettyTitle[titleLength + 1];
+
+                strncpy(prettyTitle, metadata->title, titleLength);
+                prettyTitle[titleLength] = '\0';
+
+                removeUnneededChars(prettyTitle, titleLength);
+                trim(prettyTitle, titleLength);
+
+                printTitleWithDelay(prettyTitle, ui->titleDelay, maxWidth);
         }
         else
         {
@@ -395,7 +420,7 @@ int calcElapsedBars(double elapsedSeconds, double duration, int numProgressBars)
         return (int)((elapsedSeconds / duration) * numProgressBars);
 }
 
-void printProgress(double elapsed_seconds, double total_seconds)
+void printProgress(double elapsed_seconds, double total_seconds, ma_uint32 sampleRate, int avgBitRate)
 {
         int progressWidth = 39;
         int term_w, term_h;
@@ -428,6 +453,22 @@ void printProgress(double elapsed_seconds, double total_seconds)
                total_hours, total_minutes, total_seconds_remainder,
                progress_percentage, vol);
 
+        double rate = ((float)sampleRate) / 1000;
+
+        if (term_w > progressWidth + 10)
+        {
+                if (rate == (int)rate)
+                        printf(" %dkHz", (int)rate); // No decimals
+                else
+                        printf(" %.1fkHz", rate);
+        }
+
+        if (term_w > progressWidth + 19)
+        {
+                if (avgBitRate > 0)
+                        printf(" %dkb/s ", avgBitRate);
+        }
+
         // Restore the cursor position
         printf("\033[u");
 }
@@ -447,10 +488,8 @@ void printMetadata(TagSettings const *metadata, UISettings *ui)
         printBasicMetadata(metadata, ui);
 }
 
-void printTime(double elapsedSeconds, AppState *state)
+void printTime(double elapsedSeconds, ma_uint32 sampleRate, int avgBitRate, AppState *state)
 {
-        if (appState.currentView == LIBRARY_VIEW || appState.currentView == PLAYLIST_VIEW || appState.currentView == SEARCH_VIEW)
-                return;
         if (state->uiSettings.useConfigColors)
                 setDefaultTextColor();
         else
@@ -461,7 +500,7 @@ void printTime(double elapsedSeconds, AppState *state)
         if (term_h > minHeight)
         {
                 double duration = getCurrentSongDuration();
-                printProgress(elapsedSeconds, duration);
+                printProgress(elapsedSeconds, duration, sampleRate, avgBitRate);
         }
 }
 
@@ -471,20 +510,18 @@ int calcIndentNormal(void)
         return getIndentation(textWidth - 1) - 1;
 }
 
-int calcIndentSongView(SongData *songdata)
+int calcIndentTrackView(TagSettings *metadata)
 {
-        if (songdata == NULL)
+        if (metadata == NULL)
                 return calcIndentNormal();
 
-        int titleLength = strnlen(songdata->metadata->title, METADATA_MAX_LENGTH);
-        int albumLength = strnlen(songdata->metadata->album, METADATA_MAX_LENGTH);
+        int titleLength = strnlen(metadata->title, METADATA_MAX_LENGTH);
+        int albumLength = strnlen(metadata->album, METADATA_MAX_LENGTH);
         int maxTextLength = (albumLength > titleLength) ? albumLength : titleLength;
         textWidth = (ABSOLUTE_MIN_WIDTH > preferredWidth) ? ABSOLUTE_MIN_WIDTH : preferredWidth;
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
         int maxSize = term_w - 2;
-        if (titleLength > 0 && titleLength < maxSize && titleLength > textWidth)
-                textWidth = titleLength;
         if (maxTextLength > 0 && maxTextLength < maxSize && maxTextLength > textWidth)
                 textWidth = maxTextLength;
         if (textWidth > maxSize)
@@ -501,18 +538,16 @@ void calcIndent(SongData *songdata)
         }
         else
         {
-                indent = calcIndentSongView(songdata);
+                indent = calcIndentTrackView(songdata->metadata);
         }
 }
 
-void printGlimmeringText(char *text, int textLength, char *nerdFontText, PixelData color)
+void printGlimmeringText(char *text, int textLength, char *nerdFontText, PixelData color, int indent)
 {
         int brightIndex = 0;
         PixelData vbright = increaseLuminosity(color, 120);
         PixelData bright = increaseLuminosity(color, 60);
 
-        printBlankSpaces(calcIndentNormal());
-
         while (brightIndex < textLength)
         {
                 for (int i = 0; i < textLength; i++)
@@ -582,6 +617,10 @@ void printLastRow(UISettings *ui)
 {
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
+
+        if (preferredWidth < 0 || preferredHeight < 0) // mini view
+                return;
+
         if (term_w < ABSOLUTE_MIN_WIDTH)
                 return;
 
@@ -608,19 +647,19 @@ void printLastRow(UISettings *ui)
 
         if (isPaused())
         {
-                char pauseText[] = " \u23f8";
+                char pauseText[] = " ⏸";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
                 currentLength += strnlen(pauseText, maxLength - currentLength);
         }
         else if (isStopped())
         {
-                char pauseText[] = " \u23f9";
+                char pauseText[] = " ■";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
                 currentLength += strnlen(pauseText, maxLength - currentLength);
         }
         else
         {
-                char pauseText[] = " \u25b6";
+                char pauseText[] = " ▶";
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", pauseText);
                 currentLength += strnlen(pauseText, maxLength - currentLength);
         }
@@ -631,6 +670,12 @@ void printLastRow(UISettings *ui)
                 snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", repeatText);
                 currentLength += strnlen(repeatText, maxLength - currentLength);
         }
+        else if (isRepeatListEnabled())
+        {
+                char repeatText[] = " \u27f3L";
+                snprintf(nerdFontText + currentLength, maxLength - currentLength, "%s", repeatText);
+                currentLength += strnlen(repeatText, maxLength - currentLength);
+        }
 
         if (isShuffleEnabled())
         {
@@ -653,14 +698,14 @@ void printLastRow(UISettings *ui)
                 currentLength += strnlen(rewindText, maxLength - currentLength);
         }
 
-        printf("\033[K"); // clear the line
+        printf("\033[K"); // Clear the line
 
         int indent = calcIndentNormal();
         int textLength = strnlen(text, 100);
         int randomNumber = getRandomNumber(1, 808);
 
         if (randomNumber == 808 && !ui->hideGlimmeringText)
-                printGlimmeringText(text, textLength, nerdFontText, lastRowColor);
+                printGlimmeringText(text, textLength, nerdFontText, lastRowColor, indent);
         else
         {
                 printBlankSpaces(indent);
@@ -699,21 +744,23 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
         printBlankSpaces(indent);
         printf(" - Press F2 for Playlist View:\n");
         printBlankSpaces(indent);
-        printf("     Use ↑, ↓ keys, %s, %s keys, or mouse scroll to scroll through the playlist.\n", settings->scrollUpAlt, settings->scrollDownAlt);
+        printf("   Use ↑, ↓ keys, %s, %s keys, or mouse scroll to scroll.\n", settings->scrollUpAlt, settings->scrollDownAlt);
         printBlankSpaces(indent);
-        printf("     Press Enter or middle click to play.\n");
+        printf("   Press Enter or middle click to play.\n");
         printBlankSpaces(indent);
-        printf("     Press Backspace to clear the playlist or Delete to remove a single entry.\n");
+        printf("   Press Backspace to clear the list or Delete to remove an entry.\n");
         printBlankSpaces(indent);
         printf(" - Press F3 for Library View:\n");
         printBlankSpaces(indent);
-        printf("     Use ↑, ↓ keys, %s, %s keys, or mouse scroll to scroll through the library.\n", settings->scrollUpAlt, settings->scrollDownAlt);
+        printf("   Use ↑, ↓ keys, %s, %s keys, or mouse scroll to scroll.\n", settings->scrollUpAlt, settings->scrollDownAlt);
         printBlankSpaces(indent);
-        printf("     Press Enter or middle click to add/remove songs to/from the playlist.\n");
+        printf("   Press Enter or middle click to add/remove songs.\n");
         printBlankSpaces(indent);
         printf(" - Press F4 for Track View.\n");
         printBlankSpaces(indent);
-        printf(" - Space, %s, or right click to toggle pause.\n", settings->togglePause);
+        printf(" - Space, %s, or right click to play or pause.\n", settings->togglePause);
+        printBlankSpaces(indent);
+        printf(" - Shift+s to stop.\n");
         printBlankSpaces(indent);
         printf(" - %s toggle color derived from album or from profile.\n", settings->toggleColorsDerivedFrom);
         printBlankSpaces(indent);
@@ -735,10 +782,14 @@ int showKeyBindings(SongData *songdata, AppSettings *settings, UISettings *ui)
         printBlankSpaces(indent);
         printf(" - %s to add current song to kew.m3u (run with \"kew .\").\n", settings->addToMainPlaylist);
         printBlankSpaces(indent);
-        printf(" - Esc or %s to quit.\n", settings->quit);
+        printf(" - Esc or %s to quit.\n\n", settings->quit);
+        printBlankSpaces(indent);
+        printf(" Project URL: https://github.com/ravachol/kew\n");
+        printBlankSpaces(indent);
+        printf(" Copyright © 2022-2025 Ravachol.\n");
         printf("\n");
 
-        numPrintedRows += 24;
+        numPrintedRows += 27;
 
         while (numPrintedRows < maxListSize)
         {
@@ -769,53 +820,53 @@ void toggleShowView(ViewState viewToShow)
 
 void switchToNextView(void)
 {
-        if (appState.currentView == PLAYLIST_VIEW)
-                appState.currentView = LIBRARY_VIEW;
-        else if (appState.currentView == LIBRARY_VIEW)
+        switch (appState.currentView)
         {
-                if (currentSong != NULL)
-                {
-                        appState.currentView = TRACK_VIEW;
-                }
-                else
-                {
-                        appState.currentView = SEARCH_VIEW;
-                }
-        }
-        else if (appState.currentView == TRACK_VIEW)
+        case PLAYLIST_VIEW:
+                appState.currentView = LIBRARY_VIEW;
+                break;
+        case LIBRARY_VIEW:
+                appState.currentView = (currentSong != NULL || isRadioPlaying()) ? TRACK_VIEW : SEARCH_VIEW;
+                break;
+        case TRACK_VIEW:
                 appState.currentView = SEARCH_VIEW;
-        else if (appState.currentView == SEARCH_VIEW)
+                break;
+        case SEARCH_VIEW:
                 appState.currentView = RADIOSEARCH_VIEW;
-        else if (appState.currentView == RADIOSEARCH_VIEW)
+                break;
+        case RADIOSEARCH_VIEW:
                 appState.currentView = KEYBINDINGS_VIEW;
-        else if (appState.currentView == KEYBINDINGS_VIEW)
+                break;
+        case KEYBINDINGS_VIEW:
                 appState.currentView = PLAYLIST_VIEW;
-
+                break;
+        }
         refresh = true;
 }
 
 void switchToPreviousView(void)
 {
-        if (appState.currentView == PLAYLIST_VIEW)
+        switch (appState.currentView)
+        {
+        case PLAYLIST_VIEW:
                 appState.currentView = KEYBINDINGS_VIEW;
-        else if (appState.currentView == LIBRARY_VIEW)
+                break;
+        case LIBRARY_VIEW:
                 appState.currentView = PLAYLIST_VIEW;
-        else if (appState.currentView == TRACK_VIEW)
+                break;
+        case TRACK_VIEW:
                 appState.currentView = LIBRARY_VIEW;
-        else if (appState.currentView == SEARCH_VIEW)
-                if (currentSong != NULL)
-                {
-                        appState.currentView = TRACK_VIEW;
-                }
-                else
-                {
-                        appState.currentView = LIBRARY_VIEW;
-                }
-        else if (appState.currentView == RADIOSEARCH_VIEW)
+                break;
+        case SEARCH_VIEW:
+                appState.currentView = (currentSong != NULL || isRadioPlaying()) ? TRACK_VIEW : LIBRARY_VIEW;
+                break;
+        case RADIOSEARCH_VIEW:
                 appState.currentView = SEARCH_VIEW;
-        else if (appState.currentView == KEYBINDINGS_VIEW)
+                break;
+        case KEYBINDINGS_VIEW:
                 appState.currentView = RADIOSEARCH_VIEW;
-
+                break;
+        }
         refresh = true;
 }
 
@@ -987,7 +1038,7 @@ void showSearch(SongData *songData, int *chosenRow, UISettings *ui)
         if (term_w > indent + 38 && !ui->hideHelp)
         {
                 printBlankSpaces(indent);
-                printf(" Use ↑, ↓ to choose. Enter=Accept. Alt+Enter=Play.\n\n");
+                printf(" Use ↑, ↓ to choose. Enter=Enqueue. Alt+Enter=Play.\n\n");
                 maxSearchListSize -= 2;
         }
 
@@ -1011,8 +1062,10 @@ void showRadioSearch(SongData *songData, int *chosenRow, UISettings *ui)
         if (term_w > indent + 73 && !ui->hideHelp)
         {
                 printBlankSpaces(indent);
-                printf(" Use ↑, ↓ to choose. Enter to search and then enter to accept a station.\n\n");
-                maxRadioSearchListSize -= 2;
+                printf(" Use ↑, ↓ to choose. Enter to search and then enter to accept a station.\n");
+                printBlankSpaces(indent);
+                printf(" Shift+f to add, del to remove favorites. Empty search to show favorites.\n\n");
+                maxRadioSearchListSize -= 3;
         }
 
         displayRadioSearch(maxRadioSearchListSize, indent, chosenRow, startSearchIter, ui);
@@ -1027,13 +1080,25 @@ void showPlaylist(SongData *songData, PlayList *list, int *chosenSong, int *chos
         getTermSize(&term_w, &term_h);
         maxListSize = term_h - 3;
 
+        // Setup scrolling names
+        if (getIsLongName() && isSameNameAsLastTime && updateCounter % scrollingInterval != 0)
+        {
+                updateCounter++;
+                refresh = true;
+                return;
+        }
+        else
+                refresh = false;
+
+        clearScreen();
+
         int aboutRows = printLogoAndAdjustments(songData, term_w, &(state->uiSettings), indent);
         maxListSize -= aboutRows;
 
         if (state->uiSettings.useConfigColors)
                 setTextColor(state->uiSettings.artistColor);
         else
-                setColor(&state->uiSettings);
+                setColor(&(state->uiSettings));
 
         printBlankSpaces(indent);
         printf("   ─ PLAYLIST ─\n");
@@ -1042,7 +1107,7 @@ void showPlaylist(SongData *songData, PlayList *list, int *chosenSong, int *chos
         displayPlaylist(list, maxListSize, indent, chosenSong, chosenNodeId, state->uiState.resetPlaylistDisplay, state);
 
         printErrorRow();
-        printLastRow(&state->uiSettings);
+        printLastRow(&(state->uiSettings));
 }
 
 void resetSearchResult(void)
@@ -1055,8 +1120,52 @@ void resetRadioSearchResult(void)
         chosenRadioSearchResultRow = 0;
 }
 
-void printElapsedBars(int elapsedBars, int numProgressBars)
+void printElapsedLine(int elapsedBars, int numProgressBars, PixelData color, bool useConfigColors)
 {
+        printBlankSpaces(indent);
+        printf(" ");
+        for (int i = 0; i < numProgressBars; i++)
+        {
+                if (i > elapsedBars)
+                {
+                        if (!useConfigColors)
+                        {
+                                PixelData tmp = increaseLuminosity(color, 50);
+                                printf("\033[38;2;%d;%d;%dm", tmp.r, tmp.g, tmp.b);
+                        }
+                        else
+                        {
+                                setTextColorRGB(lastRowColor.r, lastRowColor.g, lastRowColor.b);
+                        }
+                        printf("─");
+                }
+                else if (i <= elapsedBars)
+                {
+                        if (!useConfigColors)
+                        {
+                                printf("\033[38;2;%d;%d;%dm", color.r, color.g, color.b);
+                        }
+                        else
+                        {
+                                setDefaultTextColor();
+                        }
+
+                        printf("─");
+                }
+        }
+}
+
+void printElapsedBars(int elapsedBars, int numProgressBars, PixelData color, int height, bool useConfigColors)
+{
+        if (!useConfigColors)
+        {
+                PixelData tmp = increaseLuminosity(color, round(height * 4));
+                printf("\033[38;2;%d;%d;%dm", tmp.r, tmp.g, tmp.b);
+        }
+        else
+        {
+                setDefaultTextColor();
+        }
         printBlankSpaces(indent);
         printf(" ");
         for (int i = 0; i < numProgressBars; i++)
@@ -1081,7 +1190,7 @@ void printVisualizer(double elapsedSeconds, AppState *state)
         int term_w, term_h;
         getTermSize(&term_w, &term_h);
 
-        if (ui->visualizerEnabled && appState.currentView == TRACK_VIEW)
+        if (ui->visualizerEnabled)
         {
                 printf("\n");
 
@@ -1095,10 +1204,14 @@ void printVisualizer(double elapsedSeconds, AppState *state)
 #ifndef __APPLE__
                 saveCursorPosition();
 #endif
-                drawSpectrumVisualizer(ui->visualizerHeight, visualizerWidth, ui->color, indent, ui->useConfigColors);
-                printElapsedBars(calcElapsedBars(elapsedSeconds, duration, uis->numProgressBars), uis->numProgressBars);
+                drawSpectrumVisualizer(state, indent);
+
+                if (ui->progressBarType == 1)
+                        printElapsedLine(calcElapsedBars(elapsedSeconds, duration, uis->numProgressBars * 2), uis->numProgressBars * 2 - 1, ui->color, ui->useConfigColors);
+                else
+                        printElapsedBars(calcElapsedBars(elapsedSeconds, duration, uis->numProgressBars), uis->numProgressBars, ui->color, ui->visualizerHeight, ui->useConfigColors);
                 printErrorRow();
-                printLastRow(&state->uiSettings);
+                printLastRow(&(state->uiSettings));
 #ifndef __APPLE__
                 restoreCursorPosition();
                 cursorJump(1);
@@ -1147,34 +1260,6 @@ FileSystemEntry *getChosenDir(void)
         return chosenDir;
 }
 
-void processName(const char *name, char *output, int maxWidth)
-{
-        char *lastDot = strrchr(name, '.');
-        int copyLength;
-
-        if (lastDot != NULL)
-        {
-                copyLength = lastDot - name;
-                if (copyLength > maxWidth)
-                {
-                        copyLength = maxWidth;
-                }
-        }
-        else
-        {
-                copyLength = maxWidth;
-        }
-
-        if (copyLength < 0)
-                copyLength = 0;
-
-        c_strcpy(output, name, copyLength + 1);
-
-        output[copyLength] = '\0';
-        removeUnneededChars(output, copyLength);
-        trim(output, copyLength);
-}
-
 void setChosenDir(FileSystemEntry *entry)
 {
         if (entry == NULL)
@@ -1291,7 +1376,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                                 if (ui->useConfigColors)
                                                         setTextColor(ui->enqueuedColor);
                                                 else
-                                                        setColor(ui);
+                                                        setColorAndWeight(0, ui);
 
                                                 printf("\x1b[7m * ");
                                         }
@@ -1361,11 +1446,31 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                 else
                                 {
                                         filename[0] = '\0';
-                                        processName(root->name, filename, maxNameWidth - extraIndent);
+
+                                        isSameNameAsLastTime = (previousChosenLibRow == chosenLibRow);
+
+                                        if (foundChosen)
+                                        {
+                                                previousChosenLibRow = chosenLibRow;
+                                        }
+
+                                        if (!isSameNameAsLastTime)
+                                        {
+                                                resetNameScroll();
+                                        }
+
+                                        if (foundChosen)
+                                        {
+                                                processNameScroll(root->name, filename, maxNameWidth - extraIndent, isSameNameAsLastTime);
+                                        }
+                                        else
+                                        {
+                                                processName(root->name, filename, maxNameWidth - extraIndent);
+                                        }
 
                                         printf("└─ ");
 
-                                        if (foundCurrent)
+                                        if (foundCurrent && chosenLibRow != libIter)
                                         {
                                                 printf("\e[4m\e[1m");
                                         }
@@ -1396,40 +1501,22 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
 
 char *getLibraryFilePath(void)
 {
-        char *configdir = getConfigPath();
-        char *filepath = NULL;
-
-        if (configdir == NULL)
-        {
-                return NULL;
-        }
-
-        size_t configdir_length = strnlen(configdir, MAXPATHLEN);
-        size_t library_file_length = strnlen(LIBRARY_FILE, sizeof(LIBRARY_FILE));
-
-        size_t filepath_length = configdir_length + 1 + library_file_length + 1;
-
-        if (filepath_length > MAXPATHLEN)
-        {
-                free(configdir);
-                return NULL;
-        }
-
-        filepath = (char *)malloc(filepath_length);
-        if (filepath == NULL)
-        {
-                free(configdir);
-                return NULL;
-        }
-
-        snprintf(filepath, filepath_length, "%s/%s", configdir, LIBRARY_FILE);
-
-        free(configdir);
-        return filepath;
+        return getFilePath(LIBRARY_FILE);
 }
 
 void showLibrary(SongData *songData, AppState *state)
 {
+        // For scrolling names, update every nth time
+        if (getIsLongName() && isSameNameAsLastTime && updateCounter % scrollingInterval != 0)
+        {
+                refresh = true;
+                return;
+        }
+        else
+                refresh = false;
+
+        clearScreen();
+
         if (state->uiState.collapseView)
         {
                 if (previousChosenLibRow < chosenLibRow)
@@ -1486,9 +1573,9 @@ void showLibrary(SongData *songData, AppState *state)
                 printf(" Use ↑, ↓ or k, j to choose. Enter=Enqueue/Dequeue. Alt+Enter=Play.\n");
                 printBlankSpaces(indent);
 #ifndef __APPLE__
-                printf(" Pg Up and Pg Dn to scroll. Press u to update the library.\n\n");
+                printf(" Pg Up and Pg Dn to scroll. Press u to update, o to sort.\n\n");
 #else
-                printf(" Fn+Arrow Up and Fn+Arrow Down to scroll. u to update the library.\n\n");
+                printf(" Fn+Arrow Up and Fn+Arrow Down to scroll. u to update, o to sort.\n\n");
 #endif
         }
 
@@ -1520,7 +1607,7 @@ void showLibrary(SongData *songData, AppState *state)
         printErrorRow();
         printLastRow(ui);
 
-        if (refresh)
+        if (!foundChosen && refresh)
         {
                 printf("\033[1;1H");
                 clearScreen();
@@ -1528,6 +1615,170 @@ void showLibrary(SongData *songData, AppState *state)
         }
 }
 
+void createMetadataForRadio(TagSettings **metadata, RadioSearchResult *station)
+{
+        if (station)
+        {
+                *metadata = malloc(sizeof(TagSettings));
+                if (!*metadata)
+                        return;
+
+                if (station->bitrate > 0)
+                {
+                        // Reserve space for " %dkb/s"
+                        const int maxBitratePartLen = 18;
+                        size_t maxNameLen = sizeof((*metadata)->title) - maxBitratePartLen;
+
+                        // Copy a truncated name safely
+                        strncpy((*metadata)->title, station->name, maxNameLen);
+                        (*metadata)->title[maxNameLen] = '\0'; // ensure null termination
+
+                        // Now append the bitrate string safely
+                        snprintf((*metadata)->title + strlen((*metadata)->title),
+                                 sizeof((*metadata)->title) - strlen((*metadata)->title),
+                                 " %dkb/s", station->bitrate);
+                }
+                else
+                {
+                        c_strcpy((*(metadata))->title, station->name, sizeof((*(metadata))->title) - 1);
+                        (*metadata)->title[sizeof((*metadata)->title) - 1] = '\0';
+                }
+
+                (*metadata)->album[0] = '\0';
+                (*metadata)->artist[0] = '\0';
+                (*metadata)->date[0] = '\0';
+
+                calcIndentTrackView(*metadata);
+        }
+}
+
+void showTrackViewMini(SongData *songdata, AppState *state, double elapsedSeconds)
+{
+        TagSettings *metadata = NULL;
+        int avgBitRate = 0;
+
+        if (refresh)
+        {
+                printf("\n");
+
+                clearScreen();
+
+                if (currentSong == NULL && !isRadioPlaying())
+                {
+                        int term_w, term_h;
+                        getTermSize(&term_w, &term_h);
+
+                        if (term_w > 21 && term_h > 4)
+                        {
+                                for (size_t i = 0; i < sizeof(LOGO) / sizeof(LOGO[0]); i++)
+                                {
+                                        printBlankSpaces(indent);
+                                        printf("%s", LOGO[i]);
+                                }
+                                printf("\n");
+                                printBlankSpaces(indent);
+                                printf(" kew version: %s", VERSION);
+                        }
+                        return;
+                }
+                if (songdata)
+                {
+                        metadata = songdata->metadata;
+                }
+                else if (isRadioPlaying())
+                {
+                        RadioSearchResult *station = getCurrentPlayingRadioStation();
+
+                        createMetadataForRadio(&metadata, station);
+                }
+
+                calcIndentTrackView(metadata);
+
+                if (state->uiSettings.useConfigColors)
+                        setDefaultTextColor();
+                else
+                        setTextColorRGB(state->uiSettings.color.r, state->uiSettings.color.g, state->uiSettings.color.b);
+
+                if (strnlen(metadata->artist, METADATA_MAX_LENGTH) > 0 && strnlen(metadata->title, METADATA_MAX_LENGTH) > 0)
+                {
+                        printBlankSpaces(indent);
+
+                        int text_len = textWidth - 3;
+                        char combined[text_len + 1];
+                        snprintf(combined, sizeof(combined), "%s - %s", metadata->artist, metadata->title);
+
+                        printf(" %.*s\n", text_len, combined);
+                }
+                else if (strnlen(metadata->title, METADATA_MAX_LENGTH) > 0)
+                {
+                        printBlankSpaces(indent);
+                        printf(" %.*s\n", textWidth, metadata->title);
+                }
+
+                if (!songdata && metadata)
+                        free(metadata);
+                refresh = false;
+        }
+
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
+
+        bool doPrintTime = term_h > (state->uiSettings.visualizerHeight + 3);
+        bool doPrintVis = term_h > (state->uiSettings.visualizerHeight + 2);
+
+        if (songdata && doPrintTime)
+        {
+                ma_uint32 sampleRate;
+                ma_format format;
+                avgBitRate = songdata->avgBitRate;
+                getCurrentFormatAndSampleRate(&format, &sampleRate);
+                printTime(elapsedSeconds, sampleRate, avgBitRate, state);
+        }
+        if (doPrintVis)
+                printVisualizer(elapsedSeconds, state);
+}
+
+void showTrackView(SongData *songdata, AppState *state, double elapsedSeconds)
+{
+        TagSettings *metadata = NULL;
+        int avgBitRate = 0;
+
+        if (refresh)
+        {
+                if (songdata)
+                {
+                        metadata = songdata->metadata;
+                }
+                else if (isRadioPlaying())
+                {
+                        RadioSearchResult *station = getCurrentPlayingRadioStation();
+
+                        avgBitRate = station->bitrate;
+
+                        createMetadataForRadio(&metadata, station);
+                }
+
+                clearScreen();
+                printf("\n");
+                printCover(songdata, &(state->uiSettings));
+
+                printMetadata(metadata, &(state->uiSettings));
+
+                if (!songdata && metadata)
+                        free(metadata);
+                refresh = false;
+        }
+        if (songdata)
+        {
+                ma_uint32 sampleRate;
+                ma_format format;
+                avgBitRate = songdata->avgBitRate;
+                getCurrentFormatAndSampleRate(&format, &sampleRate);
+                printTime(elapsedSeconds, sampleRate, avgBitRate, state);
+        }
+        printVisualizer(elapsedSeconds, state);
+}
+
 int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings, AppState *state)
 {
         UISettings *ui = &(state->uiSettings);
@@ -1555,7 +1806,7 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
                 }
                 else
                 {
-                        if (state->currentView == TRACK_VIEW)
+                        if (state->currentView == TRACK_VIEW && !isRadioPlaying())
                         {
                                 state->currentView = LIBRARY_VIEW;
                         }
@@ -1569,9 +1820,18 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
                 calcIndent(songdata);
         }
 
-        if (preferredWidth <= 0 || preferredHeight <= 0)
-                return -1;
+        int term_w, term_h;
+        getTermSize(&term_w, &term_h);
 
+        state->uiState.miniMode = false;
+
+        if ((term_w <= 10 || term_h <= 8) || (preferredHeight <= 0 || preferredWidth <= 0))
+        {
+                state->uiState.miniMode = true;
+                showTrackViewMini(songdata, state, elapsedSeconds);
+                fflush(stdout);
+                return 0;
+        }
         if (state->currentView != PLAYLIST_VIEW)
                 state->uiState.resetPlaylistDisplay = true;
 
@@ -1585,10 +1845,8 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
         }
         else if (state->currentView == PLAYLIST_VIEW && refresh)
         {
-                clearScreen();
-                showPlaylist(songdata, originalPlaylist, &chosenRow, &uis->chosenNodeId, state);
+                showPlaylist(songdata, originalPlaylist, &chosenRow, &(uis->chosenNodeId), state);
                 state->uiState.resetPlaylistDisplay = false;
-                refresh = false;
                 fflush(stdout);
         }
         else if (state->currentView == SEARCH_VIEW && refresh)
@@ -1607,23 +1865,12 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
         }
         else if (state->currentView == LIBRARY_VIEW && refresh)
         {
-                clearScreen();
                 showLibrary(songdata, state);
-                refresh = false;
                 fflush(stdout);
         }
-        else if (state->currentView == TRACK_VIEW && songdata != NULL)
+        else if (state->currentView == TRACK_VIEW)
         {
-                if (refresh)
-                {
-                        clearScreen();
-                        printf("\n");
-                        printCover(songdata, ui);
-                        printMetadata(songdata->metadata, ui);
-                        refresh = false;
-                }
-                printTime(elapsedSeconds, state);
-                printVisualizer(elapsedSeconds, state);
+                showTrackView(songdata, state, elapsedSeconds);
                 fflush(stdout);
         }
 
