@@ -92,6 +92,7 @@ void updateLastSongSwitchTime(void)
 
 void updatePlaybackPosition(double elapsedSeconds)
 {
+#ifndef __APPLE__
         GVariantBuilder changedPropertiesBuilder;
         g_variant_builder_init(&changedPropertiesBuilder, G_VARIANT_TYPE_DICTIONARY);
         g_variant_builder_add(&changedPropertiesBuilder, "{sv}", "Position", g_variant_new_int64(llround(elapsedSeconds * G_USEC_PER_SEC)));
@@ -105,6 +106,9 @@ void updatePlaybackPosition(double elapsedSeconds)
                                       "PropertiesChanged",
                                       parameters,
                                       NULL);
+#else
+        (void)elapsedSeconds;
+#endif
 }
 
 void emitSeekedSignal(double newPositionSeconds)
@@ -181,7 +185,7 @@ void play(Node *song)
         songLoading = true;
         forceSkip = false;
 
-        // cancel starting from top
+        // Cancel starting from top
         if (waitingForPlaylist || audioData.restart)
         {
                 waitingForPlaylist = false;
@@ -213,7 +217,7 @@ void play(Node *song)
                         skipToSong(currentSong->next->id, true);
         }
 
-        updateLastSongSwitchTime();
+        resetClock();
         skip();
 }
 
@@ -238,8 +242,8 @@ void skipToBegginningOfSong(void)
 {
         if (currentSong != NULL)
         {
-                bool playSong = false;
-                skipToSong(currentSong->id, playSong);
+                seekPercentage(0);
+                emitSeekedSignal(0.0);
         }
 }
 
@@ -313,17 +317,28 @@ void togglePause(double *totalPauseSeconds, double *pauseSeconds, struct timespe
 void toggleRepeat(void)
 {
 
-        bool repeatEnabled = !isRepeatEnabled();
-        setRepeatEnabled(repeatEnabled);
+        bool repeatEnabled = isRepeatEnabled();
+        bool repeatListEnabled = isRepeatListEnabled();
 
         if (repeatEnabled)
         {
-                emitStringPropertyChanged("LoopStatus", "Track");
+                setRepeatEnabled(false);
+                setRepeatListEnabled(true);
+                emitStringPropertyChanged("LoopStatus", "List");
+        }
+        else if (repeatListEnabled)
+        {
+                setRepeatEnabled(false);
+                setRepeatListEnabled(false);
+                emitStringPropertyChanged("LoopStatus", "None");
         }
         else
         {
-                emitStringPropertyChanged("LoopStatus", "None");
+                setRepeatEnabled(true);
+                setRepeatListEnabled(false);
+                emitStringPropertyChanged("LoopStatus", "Track");
         }
+
         if (appState.currentView != TRACK_VIEW)
                 refresh = true;
 }
@@ -337,7 +352,7 @@ void addToSpecialPlaylist(void)
 
         Node *node = NULL;
 
-        if (findSelectedEntryById(specialPlaylist, id) != NULL) // song is already in list
+        if (findSelectedEntryById(specialPlaylist, id) != NULL) // Song is already in list
                 return;
 
         createNode(&node, currentSong->song.filePath, id);
@@ -390,7 +405,7 @@ void toggleShuffle(void)
                 refresh = true;
 }
 
-void toggleBlocks(AppSettings *settings, UISettings *ui)
+void toggleAscii(AppSettings *settings, UISettings *ui)
 {
         ui->coverAnsi = !ui->coverAnsi;
         c_strcpy(settings->coverAnsi, ui->coverAnsi ? "1" : "0", sizeof(settings->coverAnsi));
@@ -863,25 +878,24 @@ void moveSongUp()
 
         pthread_mutex_lock(&(playlist.mutex));
 
-        if (node != NULL && nextSong != NULL && currentSong != NULL)
+        if (node != NULL && currentSong != NULL)
         {
                 // Rebuild if current song, the next song or the song after are affected
                 if (currentSong != NULL)
                 {
-                        Node *temp = currentSong;
+                        Node *tmp = currentSong;
 
                         for (int i = 0; i < 3; i++)
                         {
-                                if (temp == NULL)
+                                if (tmp == NULL)
                                         break;
 
-                                if (temp->id == id)
+                                if (tmp->id == id)
                                 {
                                         rebuild = true;
                                 }
-                                temp = temp->next;
+                                tmp = tmp->next;
                         }
-
                 }
         }
 
@@ -935,29 +949,28 @@ void moveSongDown()
 
         pthread_mutex_lock(&(playlist.mutex));
 
-        if (node != NULL && nextSong != NULL && currentSong != NULL)
+        if (node != NULL && currentSong != NULL)
         {
                 // Rebuild if current song, the next song or the previous song are affected
                 if (currentSong != NULL)
                 {
-                        Node *temp = currentSong;
+                        Node *tmp = currentSong;
 
                         for (int i = 0; i < 2; i++)
                         {
-                                if (temp == NULL)
+                                if (tmp == NULL)
                                         break;
 
-                                if (temp->id == id)
+                                if (tmp->id == id)
                                 {
                                         rebuild = true;
                                 }
-                                temp = temp->next;
+                                tmp = tmp->next;
                         }
 
                         if (currentSong->prev != NULL && currentSong->prev->id == id)
                                 rebuild = true;
                 }
-
         }
 
         moveDownList(originalPlaylist, node);
@@ -1019,7 +1032,7 @@ void dequeueSong(FileSystemEntry *child)
 
         child->isEnqueued = 0;
 
-        // check if parent needs to be dequeued as well
+        // Check if parent needs to be dequeued as well
         bool isEnqueued = false;
 
         FileSystemEntry *ch = child->parent->children;
@@ -1227,79 +1240,88 @@ FileSystemEntry *enqueueSongs(FileSystemEntry *entry, UIState *uis)
 
 void handleRemove(void)
 {
-        if (refresh)
-                return;
-
-        if (appState.currentView != PLAYLIST_VIEW)
+        if (appState.currentView == RADIOSEARCH_VIEW)
         {
-                return;
+                RadioSearchResult *station = getCurrentRadioSearchEntry();
+
+                if (station)
+                {
+                        removeFromRadioFavorites(station);
+                }
         }
-
-        bool rebuild = false;
-
-        Node *node = findSelectedEntry(originalPlaylist, getChosenRow());
-
-        if (node == NULL)
+        else if (appState.currentView == PLAYLIST_VIEW)
         {
-                return;
-        }
 
-        Node *song = getNextSong();
-        int id = node->id;
-        int currentId = (currentSong != NULL) ? currentSong->id : -1;
+                bool rebuild = false;
 
-        if (currentId == node->id)
-        {
-                removeCurrentlyPlayingSong();
+                Node *node = findSelectedEntry(originalPlaylist, getChosenRow());
+
+                if (node == NULL)
+                {
+                        return;
+                }
+
+                Node *song = getNextSong();
+                int id = node->id;
+                int currentId = (currentSong != NULL) ? currentSong->id : -1;
+
+                if (currentId == node->id)
+                {
+                        removeCurrentlyPlayingSong();
+                }
+                else
+                {
+                        if (songToStartFrom != NULL)
+                        {
+                                songToStartFrom = getListNext(node);
+                        }
+                }
+
+                pthread_mutex_lock(&(playlist.mutex));
+
+                if (node != NULL && song != NULL && currentSong != NULL)
+                {
+                        if (strcmp(song->song.filePath, node->song.filePath) == 0 || (currentSong != NULL && currentSong->next != NULL && id == currentSong->next->id))
+                                rebuild = true;
+                }
+
+                if (node != NULL)
+                        markAsDequeued(getLibrary(), node->song.filePath);
+
+                Node *node2 = findSelectedEntryById(&playlist, id);
+
+                if (node != NULL)
+                        deleteFromList(originalPlaylist, node);
+
+                if (node2 != NULL)
+                        deleteFromList(&playlist, node2);
+
+                if (isShuffleEnabled())
+                        rebuild = true;
+
+                currentSong = findSelectedEntryById(&playlist, currentId);
+
+                if (rebuild && currentSong != NULL)
+                {
+                        node = NULL;
+                        nextSong = NULL;
+                        reshufflePlaylist();
+
+                        tryNextSong = currentSong->next;
+                        nextSongNeedsRebuilding = false;
+                        nextSong = NULL;
+                        nextSong = getListNext(currentSong);
+                        rebuildNextSong(nextSong);
+                        loadedNextSong = true;
+                }
+
+                pthread_mutex_unlock(&(playlist.mutex));
         }
         else
         {
-                if (songToStartFrom != NULL)
-                {
-                        songToStartFrom = getListNext(node);
-                }
+                return;
         }
 
-        pthread_mutex_lock(&(playlist.mutex));
-
-        if (node != NULL && song != NULL && currentSong != NULL)
-        {
-                if (strcmp(song->song.filePath, node->song.filePath) == 0 || (currentSong != NULL && currentSong->next != NULL && id == currentSong->next->id))
-                        rebuild = true;
-        }
-
-        if (node != NULL)
-                markAsDequeued(getLibrary(), node->song.filePath);
-
-        Node *node2 = findSelectedEntryById(&playlist, id);
-
-        if (node != NULL)
-                deleteFromList(originalPlaylist, node);
-
-        if (node2 != NULL)
-                deleteFromList(&playlist, node2);
-
-        if (isShuffleEnabled())
-                rebuild = true;
-
-        currentSong = findSelectedEntryById(&playlist, currentId);
-
-        if (rebuild && currentSong != NULL)
-        {
-                node = NULL;
-                nextSong = NULL;
-                reshufflePlaylist();
-
-                tryNextSong = currentSong->next;
-                nextSongNeedsRebuilding = false;
-                nextSong = NULL;
-                nextSong = getListNext(currentSong);
-                rebuildNextSong(nextSong);
-                loadedNextSong = true;
-        }
-
-        pthread_mutex_unlock(&(playlist.mutex));
-
         refresh = true;
 }
 
@@ -1333,7 +1355,7 @@ int loadDecoder(SongData *songData, bool *songDataDeleted)
         {
                 *songDataDeleted = false;
 
-                // this should only be done for the second song, as switchAudioImplementation() handles the first one
+                // This should only be done for the second song, as switchAudioImplementation() handles the first one
                 if (!loadingdata.loadingFirstDecoder)
                 {
                         if (hasBuiltinDecoder(songData->filePath))
@@ -1359,12 +1381,12 @@ int assignLoadedData(void)
         if (loadingdata.loadA)
         {
                 userData.songdataA = loadingdata.songdataA;
-                result = loadDecoder(loadingdata.songdataA, &userData.songdataADeleted);
+                result = loadDecoder(loadingdata.songdataA, &(userData.songdataADeleted));
         }
         else
         {
                 userData.songdataB = loadingdata.songdataB;
-                result = loadDecoder(loadingdata.songdataB, &userData.songdataBDeleted);
+                result = loadDecoder(loadingdata.songdataB, &(userData.songdataBDeleted));
         }
 
         return result;
@@ -1387,7 +1409,7 @@ void *songDataReaderThread(void *arg)
                 if (!userData.songdataADeleted)
                 {
                         userData.songdataADeleted = true;
-                        unloadSongData(&loadingdata->songdataA, &appState);
+                        unloadSongData(&(loadingdata->songdataA), &appState);
                 }
         }
         else
@@ -1395,7 +1417,7 @@ void *songDataReaderThread(void *arg)
                 if (!userData.songdataBDeleted)
                 {
                         userData.songdataBDeleted = true;
-                        unloadSongData(&loadingdata->songdataB, &appState);
+                        unloadSongData(&(loadingdata->songdataB), &appState);
                 }
         }
 
@@ -1506,10 +1528,29 @@ void stop(void)
 
         if (isStopped())
         {
+                skipToBegginningOfSong();
                 emitStringPropertyChanged("PlaybackStatus", "Stopped");
         }
 }
 
+int currentSort = 0;
+
+void sortLibrary(void)
+{
+        if (currentSort == 0)
+        {
+                sortFileSystemTree(library, compareFoldersByAgeFilesAlphabetically);
+                currentSort = 1;
+        }
+        else
+        {
+                sortFileSystemTree(library, compareEntryNatural);
+                currentSort = 0;
+        }
+
+        refresh = true;
+}
+
 void loadNextSong(void)
 {
         songLoading = true;
@@ -1568,14 +1609,37 @@ void resetTimeCount(void)
         totalPauseSeconds = 0.0;
 }
 
+void resetClock(void)
+{
+        resetTimeCount();
+        clock_gettime(CLOCK_MONOTONIC, &start_time);
+}
+
+void repeatList()
+{
+        waitingForPlaylist = true;
+        nextSongNeedsRebuilding = true;
+        audioData.endOfListReached = false;
+}
+
 void skipToNextSong(AppState *state)
 {
         // Stop if there is no song or no next song
         if (currentSong == NULL || currentSong->next == NULL)
         {
-                if (!isStopped() && !isPaused())
+                if (isRepeatListEnabled())
+                {
+                        currentSong = NULL;
+                }
+                else if (!isStopped() && !isPaused())
+                {
                         stop();
-                return;
+                        return;
+                }
+                else
+                {
+                        return;
+                }
         }
 
         if (songLoading || nextSongNeedsRebuilding || skipping || clearingErrors)
@@ -1592,26 +1656,17 @@ void skipToNextSong(AppState *state)
         skipping = true;
         skipOutOfOrder = false;
 
-        updateLastSongSwitchTime();
+        resetClock();
 
         skip();
 }
 
 void setCurrentSongToPrev(void)
 {
-        if (currentSong != NULL)
+        if (currentSong != NULL && currentSong->prev != NULL)
         {
-                if (isShuffleEnabled())
-                {
-                        if (currentSong->prev == NULL && currentSong->next != NULL)
-                                currentSong = currentSong->next;
-                        else
-                                return;
-                }
-                else
-                {
-                        currentSong = currentSong->prev;
-                }
+                lastPlayedId = currentSong->id;
+                currentSong = currentSong->prev;
         }
 }
 
@@ -1648,8 +1703,7 @@ void silentSwitchToPrev(AppState *state)
 
 void skipToPrevSong(AppState *state)
 {
-        // Stop if there is no song or no previous song
-        if ((currentSong == NULL || currentSong->prev == NULL) && !isShuffleEnabled())
+        if (currentSong == NULL)
         {
                 if (!isStopped() && !isPaused())
                         stop();
@@ -1666,8 +1720,16 @@ void skipToPrevSong(AppState *state)
                 return;
         }
 
+        Node *song = currentSong;
+
         setCurrentSongToPrev();
 
+        if (song == currentSong)
+        {
+                resetTimeCount();
+                updatePlaybackPosition(0); // We need to signal to mpris that the song was reset to the beginning
+        }
+
         playbackPlay(&totalPauseSeconds, &pauseSeconds);
 
         skipping = true;
@@ -1695,7 +1757,8 @@ void skipToPrevSong(AppState *state)
                 skipToPrevSong(state);
         }
 
-        updateLastSongSwitchTime();
+        resetClock();
+
         skip();
 }
 
@@ -1735,7 +1798,7 @@ void skipToNumberedSong(int songNumber)
                         skipToNumberedSong(songNumber + 1);
         }
 
-        updateLastSongSwitchTime();
+        resetClock();
         skip();
 }
 
@@ -1779,7 +1842,7 @@ void unloadSongA(AppState *state)
         if (userData.songdataADeleted == false)
         {
                 userData.songdataADeleted = true;
-                unloadSongData(&loadingdata.songdataA, state);
+                unloadSongData(&(loadingdata.songdataA), state);
                 userData.songdataA = NULL;
         }
 }
@@ -1789,7 +1852,7 @@ void unloadSongB(AppState *state)
         if (userData.songdataBDeleted == false)
         {
                 userData.songdataBDeleted = true;
-                unloadSongData(&loadingdata.songdataB, state);
+                unloadSongData(&(loadingdata.songdataB), state);
                 userData.songdataB = NULL;
         }
 }
@@ -1826,7 +1889,7 @@ void unloadPreviousSong(AppState *state)
 
 int loadFirst(Node *song, AppState *state)
 {
-        loadFirstSong(song, &state->uiSettings);
+        loadFirstSong(song, &(state->uiSettings));
 
         usingSongDataA = true;
 
@@ -1835,12 +1898,12 @@ int loadFirst(Node *song, AppState *state)
                 songHasErrors = false;
                 loadedNextSong = false;
                 currentSong = currentSong->next;
-                loadFirstSong(currentSong, &state->uiSettings);
+                loadFirstSong(currentSong, &(state->uiSettings));
         }
 
         if (songHasErrors)
         {
-                // couldn't play any of the songs
+                // Couldn't play any of the songs
                 unloadPreviousSong(state);
                 currentSong = NULL;
                 songHasErrors = false;
@@ -1858,9 +1921,9 @@ void *updateLibraryThread(void *arg)
         char *path = (char *)arg;
         int tmpDirectoryTreeEntries = 0;
 
-        FileSystemEntry *temp = createDirectoryTree(path, &tmpDirectoryTreeEntries);
+        FileSystemEntry *tmp = createDirectoryTree(path, &tmpDirectoryTreeEntries);
 
-        if (!temp)
+        if (!tmp)
         {
                 perror("createDirectoryTree");
                 pthread_mutex_unlock(&switchMutex);
@@ -1869,10 +1932,10 @@ void *updateLibraryThread(void *arg)
 
         pthread_mutex_lock(&switchMutex);
 
-        copyIsEnqueued(library, temp);
+        copyIsEnqueued(library, tmp);
 
         freeTree(library);
-        library = temp;
+        library = tmp;
         appState.uiState.numDirectoryTreeEntries = tmpDirectoryTreeEntries;
         resetChosenDir();
 
@@ -1941,7 +2004,7 @@ void createLibrary(AppSettings *settings, AppState *state)
         if (state->uiSettings.cacheLibrary > 0)
         {
                 char *libFilepath = getLibraryFilePath();
-                library = reconstructTreeFromFile(libFilepath, settings->path, &state->uiState.numDirectoryTreeEntries);
+                library = reconstructTreeFromFile(libFilepath, settings->path, &(state->uiState.numDirectoryTreeEntries));
                 free(libFilepath);
                 updateLibraryIfChangedDetected();
         }
@@ -1952,7 +2015,7 @@ void createLibrary(AppSettings *settings, AppState *state)
 
                 gettimeofday(&start, NULL);
 
-                library = createDirectoryTree(settings->path, &state->uiState.numDirectoryTreeEntries);
+                library = createDirectoryTree(settings->path, &(state->uiState.numDirectoryTreeEntries));
 
                 gettimeofday(&end, NULL);
                 long seconds = end.tv_sec - start.tv_sec;
@@ -1962,7 +2025,7 @@ void createLibrary(AppSettings *settings, AppState *state)
                 // If time to load the library was significant, ask to use cache instead
                 if (elapsed > ASK_IF_USE_CACHE_LIMIT_SECONDS)
                 {
-                        askIfCacheLibrary(&state->uiSettings);
+                        askIfCacheLibrary(&(state->uiSettings));
                 }
         }
 
@@ -2072,7 +2135,7 @@ void updateLibraryIfChangedDetected(void)
         }
 
         args->path = settings.path;
-        args->ui = &appState.uiSettings;
+        args->ui = &(appState.uiSettings);
 
         if (pthread_create(&tid, NULL, updateIfTopLevelFoldersMtimesChangedThread, (void *)args) != 0)
         {
