@@ -81,7 +81,7 @@ void toggleShuffle(void);
 
 void addToSpecialPlaylist(void);
 
-void toggleBlocks(AppSettings *settings, UISettings *ui);
+void toggleAscii(AppSettings *settings, UISettings *ui);
 
 void toggleColors(AppSettings *settings, UISettings *ui);
 
@@ -133,6 +133,8 @@ void unloadPreviousSong(AppState *state);
 
 void createLibrary(AppSettings *settings, AppState *state);
 
+void resetClock(void);
+
 void loadNextSong(void);
 
 void setCurrentSongToNext(void);
@@ -165,4 +167,10 @@ void moveSongDown();
 
 void play(Node *song);
 
+void repeatList();
+
+void skipToBegginningOfSong(void);
+
+void sortLibrary(void);
+
 #endif
