@@ -38,9 +38,14 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.coverAnsi, "0", sizeof(settings.coverAnsi));
         c_strcpy(settings.quitAfterStopping, "0", sizeof(settings.quitAfterStopping));
         c_strcpy(settings.hideGlimmeringText, "0", sizeof(settings.hideGlimmeringText));
+        c_strcpy(settings.mouseEnabled, "1", sizeof(settings.mouseEnabled));
+        c_strcpy(settings.visualizerBrailleMode, "0", sizeof(settings.visualizerBrailleMode));
+        c_strcpy(settings.tweenFactor, "0.23", sizeof(settings.tweenFactor));
+        c_strcpy(settings.tweenFactorFall, "0.13", sizeof(settings.tweenFactor));
+        c_strcpy(settings.progressBarType, "0", sizeof(settings.progressBarType));
 #ifdef __APPLE__
-        c_strcpy(settings.visualizerEnabled, "0", sizeof(settings.visualizerEnabled)); // visualizer looks wonky in default terminal
-        c_strcpy(settings.useConfigColors, "1", sizeof(settings.useConfigColors));     // colors from album look wrong in default terminal
+        c_strcpy(settings.visualizerEnabled, "0", sizeof(settings.visualizerEnabled)); // Visualizer looks wonky in default terminal
+        c_strcpy(settings.useConfigColors, "1", sizeof(settings.useConfigColors));     // Colors from album look wrong in default terminal
 #else
         c_strcpy(settings.visualizerEnabled, "1", sizeof(settings.visualizerEnabled));
         c_strcpy(settings.useConfigColors, "0", sizeof(settings.useConfigColors));
@@ -49,8 +54,8 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.hideHelp, "0", sizeof(settings.hideHelp));
         c_strcpy(settings.cacheLibrary, "-1", sizeof(settings.cacheLibrary));
         c_strcpy(settings.visualizerHeight, "5", sizeof(settings.visualizerHeight));
+        c_strcpy(settings.visualizerColorType, "0", sizeof(settings.visualizerColorType));
         c_strcpy(settings.titleDelay, "9", sizeof(settings.titleDelay));
-
         c_strcpy(settings.nextView, "\t", sizeof(settings.nextView));
         c_strcpy(settings.prevView, "[Z", sizeof(settings.prevView));
         c_strcpy(settings.volumeUp, "+", sizeof(settings.volumeUp));
@@ -121,11 +126,13 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         c_strcpy(settings.moveSongUp, "t", sizeof(settings.moveSongUp));
         c_strcpy(settings.moveSongDown, "g", sizeof(settings.moveSongDown));
         c_strcpy(settings.enqueueAndPlay, "^M", sizeof(settings.enqueueAndPlay));
+        c_strcpy(settings.hardAddToRadioFavorites, "F", sizeof(settings.hardAddToRadioFavorites));
+        c_strcpy(settings.hardStop, "S", sizeof(settings.hardStop));
+        c_strcpy(settings.sortLibrary, "o", sizeof(settings.sortLibrary));
         c_strcpy(settings.quit, "q", sizeof(settings.quit));
         c_strcpy(settings.hardQuit, "\x1B", sizeof(settings.hardQuit));
         c_strcpy(settings.hardClearPlaylist, "\b", sizeof(settings.hardClearPlaylist));
 
-
         if (pairs == NULL)
         {
                 return settings;
@@ -161,6 +168,10 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.visualizerHeight, sizeof(settings.visualizerHeight), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "visualizercolortype") == 0)
+                {
+                        snprintf(settings.visualizerColorType, sizeof(settings.visualizerColorType), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "titledelay") == 0)
                 {
                         snprintf(settings.titleDelay, sizeof(settings.titleDelay), "%s", pair->value);
@@ -261,6 +272,14 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.titleColor, sizeof(settings.titleColor), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "mouseenabled") == 0)
+                {
+                        snprintf(settings.mouseEnabled, sizeof(settings.mouseEnabled), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "visualizerbraillemode") == 0)
+                {
+                        snprintf(settings.visualizerBrailleMode, sizeof(settings.visualizerBrailleMode), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "mouseleftclickaction") == 0)
                 {
                         snprintf(settings.mouseLeftClickAction, sizeof(settings.mouseLeftClickAction), "%s", pair->value);
@@ -281,11 +300,11 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.mouseScrollDownAction, sizeof(settings.mouseScrollDownAction), "%s", pair->value);
                 }
-                else if (strcmp(lowercaseKey, "mouseshiftscrollupaction") == 0)
+                else if (strcmp(lowercaseKey, "mousealtscrollupaction") == 0)
                 {
                         snprintf(settings.mouseAltScrollUpAction, sizeof(settings.mouseAltScrollUpAction), "%s", pair->value);
                 }
-                else if (strcmp(lowercaseKey, "mouseshiftscrolldownaction") == 0)
+                else if (strcmp(lowercaseKey, "mousealtscrolldownaction") == 0)
                 {
                         snprintf(settings.mouseAltScrollDownAction, sizeof(settings.mouseAltScrollDownAction), "%s", pair->value);
                 }
@@ -352,6 +371,26 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                         if (strcmp(pair->value, "") != 0)
                                 snprintf(settings.enqueueAndPlay, sizeof(settings.enqueueAndPlay), "%s", pair->value);
                 }
+                else if (strcmp(lowercaseKey, "sort") == 0)
+                {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.sortLibrary, sizeof(settings.sortLibrary), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "tweenfactor") == 0)
+                {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.tweenFactor, sizeof(settings.tweenFactor), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "tweenfactorfall") == 0)
+                {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.tweenFactorFall, sizeof(settings.tweenFactorFall), "%s", pair->value);
+                }
+                else if (strcmp(lowercaseKey, "progressbartype") == 0)
+                {
+                        if (strcmp(pair->value, "") != 0)
+                                snprintf(settings.progressBarType, sizeof(settings.progressBarType), "%s", pair->value);
+                }
                 else if (strcmp(lowercaseKey, "showkeysalt") == 0 && strcmp(pair->value, "B") != 0)
                 {
                         // We need to prevent the previous key B or else config files wont get updated
@@ -464,7 +503,7 @@ void mapSettingsToKeys(AppSettings *settings, UISettings *ui, EventMapping *mapp
         mappings[9] = (EventMapping){settings->hardQuit, EVENT_QUIT};
         mappings[10] = (EventMapping){settings->toggleShuffle, EVENT_SHUFFLE};
         mappings[11] = (EventMapping){settings->toggleVisualizer, EVENT_TOGGLEVISUALIZER};
-        mappings[12] = (EventMapping){settings->toggleAscii, EVENT_TOGGLEBLOCKS};
+        mappings[12] = (EventMapping){settings->toggleAscii, EVENT_TOGGLEASCII};
         mappings[13] = (EventMapping){settings->switchNumberedSong, EVENT_GOTOSONG};
         mappings[14] = (EventMapping){settings->seekBackward, EVENT_SEEKBACK};
         mappings[15] = (EventMapping){settings->seekForward, EVENT_SEEKFORWARD};
@@ -514,6 +553,9 @@ void mapSettingsToKeys(AppSettings *settings, UISettings *ui, EventMapping *mapp
         mappings[59] = (EventMapping){settings->moveSongUp, EVENT_MOVESONGUP};
         mappings[60] = (EventMapping){settings->moveSongDown, EVENT_MOVESONGDOWN};
         mappings[61] = (EventMapping){settings->enqueueAndPlay, EVENT_ENQUEUEANDPLAY};
+        mappings[62] = (EventMapping){settings->hardStop, EVENT_STOP};
+        mappings[63] = (EventMapping){settings->hardAddToRadioFavorites, EVENT_ADDTORADIOFAVORITES};
+        mappings[64] = (EventMapping){settings->sortLibrary, EVENT_SORTLIBRARY};
 }
 
 char *getConfigFilePath(char *configdir)
@@ -591,6 +633,52 @@ enum EventType getMouseAction(int num)
         return value;
 }
 
+int mkdir_p(const char *path, mode_t mode)
+{
+        if (path == NULL)
+                return -1;
+
+        if (path[0] == '~')
+        {
+                // Just try a plain mkdir if there's a tilde
+                if (mkdir(path, mode) == -1)
+                {
+                        if (errno != EEXIST)
+                                return -1;
+                }
+                return 0;
+        }
+
+        char tmp[PATH_MAX];
+        char *p = NULL;
+        size_t len;
+
+        snprintf(tmp, sizeof(tmp), "%s", path);
+        len = strlen(tmp);
+        if (len > 0 && tmp[len - 1] == '/')
+                tmp[len - 1] = 0;
+
+        for (p = tmp + 1; *p; p++)
+        {
+                if (*p == '/')
+                {
+                        *p = 0;
+                        if (mkdir(tmp, mode) == -1)
+                        {
+                                if (errno != EEXIST)
+                                        return -1;
+                        }
+                        *p = '/';
+                }
+        }
+        if (mkdir(tmp, mode) == -1)
+        {
+                if (errno != EEXIST)
+                        return -1;
+        }
+        return 0;
+}
+
 void getConfig(AppSettings *settings, UISettings *ui)
 {
         int pair_count;
@@ -600,7 +688,7 @@ void getConfig(AppSettings *settings, UISettings *ui)
         struct stat st = {0};
         if (stat(configdir, &st) == -1)
         {
-                if (mkdir(configdir, 0700) != 0)
+                if (mkdir_p(configdir, 0700) != 0)
                 {
                         perror("mkdir");
                         exit(1);
@@ -609,7 +697,7 @@ void getConfig(AppSettings *settings, UISettings *ui)
 
         char *filepath = getConfigFilePath(configdir);
 
-        KeyValuePair *pairs = readKeyValuePairs(filepath, &pair_count, &ui->lastTimeAppRan);
+        KeyValuePair *pairs = readKeyValuePairs(filepath, &pair_count, &(ui->lastTimeAppRan));
 
         free(filepath);
         *settings = constructAppSettings(pairs, pair_count);
@@ -621,75 +709,93 @@ void getConfig(AppSettings *settings, UISettings *ui)
         ui->useConfigColors = (settings->useConfigColors[0] == '1');
         ui->quitAfterStopping = (settings->quitAfterStopping[0] == '1');
         ui->hideGlimmeringText = (settings->hideGlimmeringText[0] == '1');
+        ui->mouseEnabled = (settings->mouseEnabled[0] == '1');
+        ui->visualizerBrailleMode = (settings->visualizerBrailleMode[0] == '1');
         ui->hideLogo = (settings->hideLogo[0] == '1');
         ui->hideHelp = (settings->hideHelp[0] == '1');
 
-        int temp = getNumber(settings->color);
-        if (temp >= 0)
-                ui->mainColor = temp;
+        int tmp = getNumber(settings->color);
+        if (tmp >= 0)
+                ui->mainColor = tmp;
 
-        temp = getNumber(settings->artistColor);
-        if (temp >= 0)
-                ui->artistColor = temp;
+        tmp = getNumber(settings->artistColor);
+        if (tmp >= 0)
+                ui->artistColor = tmp;
 
-        temp = getNumber(settings->enqueuedColor);
-        if (temp >= 0)
-                ui->enqueuedColor = temp;
+        tmp = getNumber(settings->enqueuedColor);
+        if (tmp >= 0)
+                ui->enqueuedColor = tmp;
 
-        temp = getNumber(settings->titleColor);
-        if (temp >= 0)
-                ui->titleColor = temp;
+        tmp = getNumber(settings->titleColor);
+        if (tmp >= 0)
+                ui->titleColor = tmp;
 
-        temp = getNumber(settings->mouseLeftClickAction);
-        enum EventType tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseLeftClickAction = tempEvent;
+        tmp = getNumber(settings->mouseLeftClickAction);
+        enum EventType tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseLeftClickAction = tmpEvent;
 
-        temp = getNumber(settings->mouseMiddleClickAction);
-        tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseMiddleClickAction = tempEvent;
+        tmp = getNumber(settings->mouseMiddleClickAction);
+        tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseMiddleClickAction = tmpEvent;
 
-        temp = getNumber(settings->mouseRightClickAction);
-        tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseRightClickAction = tempEvent;
+        tmp = getNumber(settings->mouseRightClickAction);
+        tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseRightClickAction = tmpEvent;
 
-        temp = getNumber(settings->mouseScrollUpAction);
-        tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseScrollUpAction = tempEvent;
+        tmp = getNumber(settings->mouseScrollUpAction);
+        tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseScrollUpAction = tmpEvent;
 
-        temp = getNumber(settings->mouseScrollDownAction);
-        tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseScrollDownAction = tempEvent;
+        tmp = getNumber(settings->mouseScrollDownAction);
+        tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseScrollDownAction = tmpEvent;
 
-        temp = getNumber(settings->mouseAltScrollUpAction);
-        tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseAltScrollUpAction = tempEvent;
+        tmp = getNumber(settings->mouseAltScrollUpAction);
+        tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseAltScrollUpAction = tmpEvent;
 
-        temp = getNumber(settings->mouseAltScrollDownAction);
-        tempEvent = getMouseAction(temp);
-        if (temp >= 0)
-                ui->mouseAltScrollDownAction = tempEvent;
+        tmp = getNumber(settings->mouseAltScrollDownAction);
+        tmpEvent = getMouseAction(tmp);
+        if (tmp >= 0)
+                ui->mouseAltScrollDownAction = tmpEvent;
 
-        temp = getNumber(settings->visualizerHeight);
-        if (temp > 0)
-                ui->visualizerHeight = temp;
+        tmp = getNumber(settings->visualizerHeight);
+        if (tmp > 0)
+                ui->visualizerHeight = tmp;
 
-        temp = getNumber(settings->titleDelay);
-        if (temp >= 0)
-                ui->titleDelay = temp;
+        tmp = getNumber(settings->visualizerColorType);
+        if (tmp >= 0)
+                ui->visualizerColorType = tmp;
 
-        temp = getNumber(settings->lastVolume);
-        if (temp >= 0)
-                setVolume(temp);
+        tmp = getNumber(settings->progressBarType);
+        if (tmp >= 0)
+                ui->progressBarType = tmp;
 
-        temp = getNumber(settings->cacheLibrary);
-        if (temp >= 0)
-                ui->cacheLibrary = temp;
+        tmp = getNumber(settings->titleDelay);
+        if (tmp >= 0)
+                ui->titleDelay = tmp;
+
+        tmp = getNumber(settings->lastVolume);
+        if (tmp >= 0)
+                setVolume(tmp);
+
+        tmp = getNumber(settings->cacheLibrary);
+        if (tmp >= 0)
+                ui->cacheLibrary = tmp;
+
+        float tmpFloat = getFloat(settings->tweenFactor);
+        if (tmpFloat >= 0.0f)
+                ui->tweenFactor = tmpFloat;
+
+        tmpFloat = getFloat(settings->tweenFactorFall);
+        if (tmpFloat >= 0.0f)
+                ui->tweenFactorFall  = tmpFloat;
 
         getMusicLibraryPath(settings->path);
         free(configdir);
@@ -726,6 +832,10 @@ void setConfig(AppSettings *settings, UISettings *ui)
                 ui->quitAfterStopping ? c_strcpy(settings->quitAfterStopping, "1", sizeof(settings->quitAfterStopping)) : c_strcpy(settings->quitAfterStopping, "0", sizeof(settings->quitAfterStopping));
         if (settings->hideGlimmeringText[0] == '\0')
                 ui->hideGlimmeringText ? c_strcpy(settings->hideGlimmeringText, "1", sizeof(settings->hideGlimmeringText)) : c_strcpy(settings->hideGlimmeringText, "0", sizeof(settings->hideGlimmeringText));
+        if (settings->mouseEnabled[0] == '\0')
+                ui->mouseEnabled ? c_strcpy(settings->mouseEnabled, "1", sizeof(settings->mouseEnabled)) : c_strcpy(settings->mouseEnabled, "0", sizeof(settings->mouseEnabled));
+        if (settings->visualizerBrailleMode[0] == '\0')
+                ui->visualizerBrailleMode ? c_strcpy(settings->visualizerBrailleMode, "1", sizeof(settings->visualizerBrailleMode)) : c_strcpy(settings->visualizerBrailleMode, "0", sizeof(settings->visualizerBrailleMode));
         if (settings->hideLogo[0] == '\0')
                 ui->hideLogo ? c_strcpy(settings->hideLogo, "1", sizeof(settings->hideLogo)) : c_strcpy(settings->hideLogo, "0", sizeof(settings->hideLogo));
         if (settings->hideHelp[0] == '\0')
@@ -733,18 +843,22 @@ void setConfig(AppSettings *settings, UISettings *ui)
 
         if (settings->visualizerHeight[0] == '\0')
                 snprintf(settings->visualizerHeight, sizeof(settings->visualizerHeight), "%d", ui->visualizerHeight);
+        if (settings->tweenFactor[0] == '\0')
+                snprintf(settings->tweenFactor, sizeof(settings->tweenFactor), "%.2f", ui->tweenFactor);
+        if (settings->tweenFactorFall[0] == '\0')
+                snprintf(settings->tweenFactorFall, sizeof(settings->tweenFactorFall), "%.2f", ui->tweenFactorFall);
+        if (settings->visualizerColorType[0] == '\0')
+                snprintf(settings->visualizerColorType, sizeof(settings->visualizerColorType), "%d", ui->visualizerColorType);
+        if (settings->progressBarType[0] == '\0')
+                snprintf(settings->progressBarType, sizeof(settings->progressBarType), "%d", ui->progressBarType);
         if (settings->titleDelay[0] == '\0')
                 snprintf(settings->titleDelay, sizeof(settings->titleDelay), "%d", ui->titleDelay);
         if (settings->cacheLibrary[0] == '\0')
                 snprintf(settings->cacheLibrary, sizeof(settings->cacheLibrary), "%d", ui->cacheLibrary);
 
-        if (settings->lastVolume[0] == '\0')
-        {
-                int currentVolume = getCurrentVolume();
-                currentVolume = (currentVolume <= 0) ? 10 : currentVolume;
-                snprintf(settings->lastVolume, sizeof(settings->lastVolume), "%d", currentVolume);
-        }
-
+        int currentVolume = getCurrentVolume();
+        currentVolume = (currentVolume <= 0) ? 10 : currentVolume;
+        snprintf(settings->lastVolume, sizeof(settings->lastVolume), "%d", currentVolume);
 
         if (settings->color[0] == '\0')
                 snprintf(settings->color, sizeof(settings->color), "%d", ui->mainColor);
@@ -773,51 +887,80 @@ void setConfig(AppSettings *settings, UISettings *ui)
         // Write the settings to the file
         fprintf(file, "# Make sure that kew is closed before editing this file in order for changes to take effect.\n\n");
 
+        fprintf(file, "[miscellaneous]\n");
         fprintf(file, "path=%s\n", settings->path);
-        fprintf(file, "coverEnabled=%s\n", settings->coverEnabled);
-        fprintf(file, "coverAnsi=%s\n", settings->coverAnsi);
-        fprintf(file, "visualizerEnabled=%s\n", settings->visualizerEnabled);
-        fprintf(file, "visualizerHeight=%s\n", settings->visualizerHeight);
-        fprintf(file, "useConfigColors=%s\n", settings->useConfigColors);
+        fprintf(file, "version=%s\n", VERSION);
         fprintf(file, "allowNotifications=%s\n", settings->allowNotifications);
         fprintf(file, "hideLogo=%s\n", settings->hideLogo);
         fprintf(file, "hideHelp=%s\n", settings->hideHelp);
-        fprintf(file, "lastVolume=%s\n", settings->lastVolume);
+        fprintf(file, "lastVolume=%s\n\n", settings->lastVolume);
 
-        fprintf(file, "\n# Cache: Set to 1 to use cache of the music library directory tree for faster startup times.\n");
-        fprintf(file, "cacheLibrary=%s\n", settings->cacheLibrary);
+        fprintf(file, "# Cache: Set to 1 to use cache of the music library directory tree for faster startup times.\n");
+        fprintf(file, "cacheLibrary=%s\n\n", settings->cacheLibrary);
 
-        fprintf(file, "\n# Delay when drawing title in track view, set to 0 to have no delay.\n");
-        fprintf(file, "titleDelay=%s\n", settings->titleDelay);
+        fprintf(file, "# Delay when drawing title in track view, set to 0 to have no delay.\n");
+        fprintf(file, "titleDelay=%s\n\n", settings->titleDelay);
 
-        fprintf(file, "\n# Same as '--quitonstop' flag, exits after playing the whole playlist.\n");
-        fprintf(file, "quitOnStop=%s\n", settings->quitAfterStopping);
+        fprintf(file, "# Same as '--quitonstop' flag, exits after playing the whole playlist.\n");
+        fprintf(file, "quitOnStop=%s\n\n", settings->quitAfterStopping);
 
-        fprintf(file, "\n# Glimmering text on the bottom row.\n");
-        fprintf(file, "hideGlimmeringText=%s\n", settings->hideGlimmeringText);
+        fprintf(file, "# Glimmering text on the bottom row.\n");
+        fprintf(file, "hideGlimmeringText=%s\n\n", settings->hideGlimmeringText);
 
-        fprintf(file, "\n# Color values are 0=Black, 1=Red, 2=Green, 3=Yellow, 4=Blue, 5=Magenta, 6=Cyan, 7=White\n");
+        fprintf(file, "[visualizer]\n");
+        fprintf(file, "visualizerEnabled=%s\n", settings->visualizerEnabled);
+        fprintf(file, "visualizerHeight=%s\n", settings->visualizerHeight);
+        fprintf(file, "visualizerBrailleMode=%s\n\n", settings->visualizerBrailleMode);
+
+        fprintf(file, "# How colors are laid out in the spectrum visualizer. 0=default, 1=brightness depending on bar height, 2=reversed.\n");
+        fprintf(file, "visualizerColorType=%s\n\n", settings->visualizerColorType);
+
+        fprintf(file, "# How fast the visualizer moves (higher values = faster) Normal values: 0.23 and 0.13.\n");
+        fprintf(file, "tweenFactor=%s\n", settings->tweenFactor);
+        fprintf(file, "tweenFactorFall=%s\n\n", settings->tweenFactorFall);
+
+        fprintf(file, "# How the progress bar looks. 0=Dots, 1=Line\n");
+        fprintf(file, "progressBarType=%s\n\n", settings->progressBarType);
+
+        fprintf(file, "[colors]\n\n");
+
+        fprintf(file, "# Use the configuration file colors below\n");
+        fprintf(file, "useConfigColors=%s\n\n", settings->useConfigColors);
+
+        fprintf(file, "# Color values are 0=Black, 1=Red, 2=Green, 3=Yellow, 4=Blue, 5=Magenta, 6=Cyan, 7=White\n");
         fprintf(file, "# These mostly affect the library view.\n\n");
-        fprintf(file, "# Logo color:\n");
-        fprintf(file, "color=%s\n", settings->color);
-        fprintf(file, "# Header color in library view:\n");
-        fprintf(file, "artistColor=%s\n", settings->artistColor);
-        fprintf(file, "# Now playing song text in library view:\n");
-        fprintf(file, "titleColor=%s\n", settings->titleColor);
-        fprintf(file, "# Color of enqueued songs in library view:\n");
-        fprintf(file, "enqueuedColor=%s\n", settings->enqueuedColor);
 
-        fprintf(file, "\n# Mouse actions are 0=none, 1=select song, 2=toggle pause, 3=scroll up, 4=scroll down, 5=seek forward, 6=seek backward, 7=volume up, 8=volume down, 9=switch to next view, 10=switch to previous view\n");
+        fprintf(file, "# Logo color:\n");
+        fprintf(file, "color=%s\n\n", settings->color);
+
+        fprintf(file, "# Header color in library view:\n");
+        fprintf(file, "artistColor=%s\n\n", settings->artistColor);
+
+        fprintf(file, "# Now playing song text in library view:\n");
+        fprintf(file, "titleColor=%s\n\n", settings->titleColor);
+
+        fprintf(file, "# Color of enqueued songs in library view:\n");
+        fprintf(file, "enqueuedColor=%s\n\n", settings->enqueuedColor);
+
+        fprintf(file, "[track cover]\n");
+        fprintf(file, "coverEnabled=%s\n", settings->coverEnabled);
+        fprintf(file, "coverAnsi=%s\n\n", settings->coverAnsi);
+
+        fprintf(file, "[mouse]\n");
+        fprintf(file, "mouseEnabled=%s\n\n", settings->mouseEnabled);
+
+        fprintf(file, "# Mouse actions are 0=none, 1=select song, 2=toggle pause, 3=scroll up, 4=scroll down, 5=seek forward, 6=seek backward, 7=volume up, 8=volume down, 9=switch to next view, 10=switch to previous view\n");
         fprintf(file, "mouseLeftClickAction=%s\n", settings->mouseLeftClickAction);
         fprintf(file, "mouseMiddleClickAction=%s\n", settings->mouseMiddleClickAction);
         fprintf(file, "mouseRightClickAction=%s\n", settings->mouseRightClickAction);
         fprintf(file, "mouseScrollUpAction=%s\n", settings->mouseScrollUpAction);
-        fprintf(file, "mouseScrollDownAction=%s\n", settings->mouseScrollDownAction);
+        fprintf(file, "mouseScrollDownAction=%s\n\n", settings->mouseScrollDownAction);
+
         fprintf(file, "# Mouse action when using mouse scroll + alt\n");
         fprintf(file, "mouseAltScrollUpAction=%s\n", settings->mouseAltScrollUpAction);
-        fprintf(file, "mouseAltScrollDownAction=%s\n", settings->mouseAltScrollDownAction);
+        fprintf(file, "mouseAltScrollDownAction=%s\n\n", settings->mouseAltScrollDownAction);
 
-        fprintf(file, "\n# Key Bindings:\n\n");
+        fprintf(file, "[key bindings]\n");
         fprintf(file, "volumeUp=%s\n", settings->volumeUp);
         fprintf(file, "volumeUpAlt=%s\n", settings->volumeUpAlt);
         fprintf(file, "volumeDown=%s\n", settings->volumeDown);
@@ -840,16 +983,17 @@ void setConfig(AppSettings *settings, UISettings *ui)
         fprintf(file, "moveSongUp=%s\n", settings->moveSongUp);
         fprintf(file, "moveSongDown=%s\n", settings->moveSongDown);
         fprintf(file, "enqueueAndPlay=%s\n", settings->enqueueAndPlay);
+        fprintf(file, "sortLibrary=%s\n", settings->sortLibrary);
+        fprintf(file, "quit=%s\n\n", settings->quit);
 
-        fprintf(file, "\n# Alt keys for the different main views, normally F2-F7:\n");
+        fprintf(file, "# Alt keys for the different main views, normally F2-F7:\n");
         fprintf(file, "showPlaylistAlt=%s\n", settings->showPlaylistAlt);
         fprintf(file, "showLibraryAlt=%s\n", settings->showLibraryAlt);
         fprintf(file, "showTrackAlt=%s\n", settings->showTrackAlt);
         fprintf(file, "showSearchAlt=%s\n", settings->showSearchAlt);
-        fprintf(file, "showRadioSearchAlt=%s\n\n", settings->showRadioSearchAlt);
+        fprintf(file, "showRadioSearchAlt=%s\n", settings->showRadioSearchAlt);
         fprintf(file, "showKeysAlt=%s\n\n", settings->showKeysAlt);
 
-        fprintf(file, "quit=%s\n\n", settings->quit);
         fprintf(file, "# For special keys use terminal codes: OS, for F4 for instance. This can depend on the terminal.\n");
         fprintf(file, "# You can find out the codes for the keys by using tools like showkey.\n");
         fprintf(file, "# For special keys, see the key value after the bracket \"[\" after typing \"showkey -a\" in the terminal and then pressing a key you want info about.\n");
