@@ -29,16 +29,18 @@ typedef struct
 #ifndef TAGSETTINGS_STRUCT
 #define TAGSETTINGS_STRUCT
 
-#define METADATA_MAX_SIZE 256
+#define METADATA_MAX_LENGTH 256
 
-typedef struct
-{
-        char title[METADATA_MAX_SIZE];
-        char artist[METADATA_MAX_SIZE];
-        char album_artist[METADATA_MAX_SIZE];
-        char album[METADATA_MAX_SIZE];
-        char date[METADATA_MAX_SIZE];
-} TagSettings;
+        typedef struct
+        {
+                char title[METADATA_MAX_LENGTH];
+                char artist[METADATA_MAX_LENGTH];
+                char album_artist[METADATA_MAX_LENGTH];
+                char album[METADATA_MAX_LENGTH];
+                char date[METADATA_MAX_LENGTH];
+                double replaygainTrack;
+                double replaygainAlbum;
+        } TagSettings;
 
 #endif
 
@@ -55,6 +57,7 @@ typedef struct
         unsigned char blue;
         TagSettings *metadata;
         unsigned char *cover;
+        int avgBitRate;
         int coverWidth;
         int coverHeight;
         double duration;
