@@ -17,6 +17,8 @@ ma_context context;
 
 bool isContextInitialized = false;
 
+bool tryAgain = false;
+
 UserData userData;
 
 ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
@@ -45,7 +47,7 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 pAudioData->format = first->outputFormat;
                 pAudioData->channels = first->outputChannels;
                 pAudioData->sampleRate = first->outputSampleRate;
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
         }
         else if (pathEndsWith(filePath, "opus"))
         {
@@ -54,8 +56,8 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_libopus *first = getFirstOpusDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_libopus_ds_get_data_format(first, &pAudioData->format, &pAudioData->channels, &pAudioData->sampleRate, channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                ma_libopus_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -67,8 +69,8 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_libvorbis *first = getFirstVorbisDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_libvorbis_ds_get_data_format(first, &pAudioData->format, &pAudioData->channels, &pAudioData->sampleRate, channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                ma_libvorbis_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -82,8 +84,8 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 m4a_decoder *first = getFirstM4aDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                m4a_decoder_ds_get_data_format(first, &pAudioData->format, &pAudioData->channels, &pAudioData->sampleRate, channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                m4a_decoder_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -272,7 +274,29 @@ bool validFilePath(char *filePath)
         return true;
 }
 
-bool tryAgain = false;
+long long getFileSize(const char *filename)
+{
+        struct stat st;
+        if (stat(filename, &st) == 0)
+        {
+                return (long long)st.st_size;
+        }
+        else
+        {
+                return -1;
+        }
+}
+
+int calcAvgBitRate(double duration, const char *filePath)
+{
+        long long fileSize = getFileSize(filePath); // in bytes
+        int avgBitRate = 0;
+
+        if (duration > 0.0)
+                avgBitRate = (int)((fileSize * 8.0) / duration / 1000.0); // use 1000 for kbps
+
+        return avgBitRate;
+}
 
 int switchAudioImplementation(void)
 {
@@ -336,6 +360,18 @@ int switchAudioImplementation(void)
                                                        channels == decoder->outputChannels &&
                                                        format == decoder->outputFormat));
 
+                if (pathEndsWith(filePath, ".mp3") && userData.currentSongData)
+                {
+                        int avgBitRate = calcAvgBitRate(userData.currentSongData->duration, filePath);
+
+                        if (avgBitRate > 320)
+                                avgBitRate = 320;
+
+                        userData.currentSongData->avgBitRate = audioData.avgBitRate = avgBitRate;
+                }
+                else
+                        audioData.avgBitRate = 0;
+
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == BUILTIN))
                 {
                         setImplSwitchReached();
@@ -349,6 +385,8 @@ int switchAudioImplementation(void)
                         resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+
                         int result = builtin_createAudioDevice(&userData, getDevice(), &context, &builtin_file_data_source_vtable);
 
                         if (result < 0)
@@ -401,6 +439,9 @@ int switchAudioImplementation(void)
                         resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+                        audioData.avgBitRate = 0;
+
                         int result = opus_createAudioDevice(&userData, getDevice(), &context);
 
                         if (result < 0)
@@ -440,6 +481,11 @@ int switchAudioImplementation(void)
                                                        channels == nChannels &&
                                                        sampleRate == nSampleRate));
 
+                if (userData.currentSongData)
+                        userData.currentSongData->avgBitRate = audioData.avgBitRate = calcAvgBitRate(userData.currentSongData->duration, filePath);
+                else
+                        audioData.avgBitRate = 0;
+
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == VORBIS))
                 {
                         setImplSwitchReached();
@@ -453,6 +499,8 @@ int switchAudioImplementation(void)
                         resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+
                         int result = vorbis_createAudioDevice(&userData, getDevice(), &context);
 
                         if (result < 0)
@@ -481,11 +529,12 @@ int switchAudioImplementation(void)
                 ma_uint32 nSampleRate;
                 ma_uint32 nChannels;
                 ma_format nFormat;
+                int avgBitRate;
                 ma_channel nChannelMap[MA_MAX_CHANNELS];
                 m4a_decoder *decoder = getCurrentM4aDecoder();
                 k_m4adec_filetype fileType = k_unknown;
 
-                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap, &fileType);
+                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap, &avgBitRate, &fileType);
 
                 if (decoder != NULL)
                         m4a_decoder_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
@@ -493,8 +542,11 @@ int switchAudioImplementation(void)
                 bool sameFormat = (decoder != NULL && (format == decoder->format &&
                                                        channels == nChannels &&
                                                        sampleRate == nSampleRate &&
-                                                        decoder->fileType == fileType &&
-                                                        decoder->fileType != k_rawAAC));
+                                                       decoder->fileType == fileType &&
+                                                       decoder->fileType != k_rawAAC));
+
+                if (userData.currentSongData)
+                        userData.currentSongData->avgBitRate = audioData.avgBitRate = avgBitRate;
 
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == M4A))
                 {
@@ -509,6 +561,8 @@ int switchAudioImplementation(void)
                         resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+
                         int result = m4a_createAudioDevice(&userData, getDevice(), &context);
 
                         if (result < 0)
