@@ -96,14 +96,44 @@ ma_data_source_vtable builtin_file_data_source_vtable = {
     builtin_file_data_source_get_cursor,
     builtin_file_data_source_get_length,
     builtin_file_data_source_set_looping,
-    0 // flags
+    0 // Flags
 };
 
+double dbToLinear(double db)
+{
+        return pow(10.0, db / 20.0);
+}
+
 void builtin_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_uint64 frameCount, ma_uint64 *pFramesRead)
 {
         AudioData *audioData = (AudioData *)pDataSource;
         ma_uint64 framesRead = 0;
 
+        // Convert ReplayGain dB values to linear gain factors
+        double gainDb = 0.0; // Default to 0 dB (no gain)
+        bool gainAvailable = false;
+
+        if ((!audioData->pUserData->songdataADeleted && audioData->pUserData->currentSongData == audioData->pUserData->songdataA) ||
+            (!audioData->pUserData->songdataBDeleted && audioData->pUserData->currentSongData == audioData->pUserData->songdataB))
+        {
+                if (audioData->pUserData->currentSongData->metadata->replaygainTrack > -50.0)
+                {
+                        gainDb = audioData->pUserData->currentSongData->metadata->replaygainTrack;
+                        gainAvailable = true;
+                }
+                else if (audioData->pUserData->currentSongData->metadata->replaygainAlbum > -50.0)
+                {
+                        gainDb = audioData->pUserData->currentSongData->metadata->replaygainAlbum;
+                        gainAvailable = true;
+                }
+        }
+
+        double totalGainFactor = 1.0;
+        if (gainAvailable)
+        {
+                totalGainFactor = dbToLinear(gainDb);
+        }
+
         while (framesRead < frameCount)
         {
                 ma_uint64 remainingFrames = frameCount - framesRead;
@@ -130,12 +160,12 @@ void builtin_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_u
 
                 if ((getCurrentImplementationType() != BUILTIN && !isSkipToNext()))
                 {
-                        pthread_mutex_unlock(&dataSourceMutex);                        
+                        pthread_mutex_unlock(&dataSourceMutex);
                         return;
                 }
 
                 if (audioData->totalFrames == 0)
-                        ma_data_source_get_length_in_pcm_frames(decoder, &audioData->totalFrames);
+                        ma_data_source_get_length_in_pcm_frames(decoder, &(audioData->totalFrames));
 
                 if (isSeekRequested())
                 {
@@ -166,7 +196,7 @@ void builtin_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_u
                 ma_result result;
 
                 if (firstDecoder == NULL)
-                {                       
+                {
                         pthread_mutex_unlock(&dataSourceMutex);
                         return;
                 }
@@ -176,10 +206,80 @@ void builtin_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_u
                         pthread_mutex_unlock(&dataSourceMutex);
                         return;
                 }
-                
+
                 result = ma_data_source_read_pcm_frames(firstDecoder, (ma_int32 *)pFramesOut + framesRead * audioData->channels, remainingFrames, &framesToRead);
                 ma_data_source_get_cursor_in_pcm_frames(decoder, &cursor);
 
+                float *frames = (float *)pFramesOut + framesRead * audioData->channels;
+
+                // Apply replay gain
+                if (totalGainFactor != 1.0)
+                {
+                        switch (audioData->format)
+                        {
+                        case ma_format_f32:
+                                for (ma_uint64 i = 0; i < framesToRead; ++i)
+                                {
+                                        for (int ch = 0; ch < (int)audioData->channels; ++ch)
+                                        {
+                                                ma_uint64 frameIndex = i * audioData->channels + ch;
+                                                float originalSample = frames[frameIndex];
+                                                double sample = (double)originalSample;
+
+                                                sample *= totalGainFactor;
+                                                frames[frameIndex] = (float)sample;
+                                        }
+                                }
+                                break;
+
+                        case ma_format_s16:
+                                for (ma_uint64 i = 0; i < framesToRead; ++i)
+                                {
+                                        for (int ch = 0; ch < (int)audioData->channels; ++ch)
+                                        {
+                                                ma_uint64 frameIndex = i * audioData->channels + ch;
+                                                ma_int16 originalSample = frames[frameIndex];
+                                                double sample = (double)originalSample;
+
+                                                sample *= totalGainFactor;
+
+                                                if (sample > 32767.0)
+                                                        sample = 32767.0;
+                                                else if (sample < -32768.0)
+                                                        sample = -32768.0;
+
+                                                frames[frameIndex] = (ma_int16)sample;
+                                        }
+                                }
+                                break;
+
+                        case ma_format_s32:
+                                for (ma_uint64 i = 0; i < framesToRead; ++i)
+                                {
+                                        for (int ch = 0; ch < (int)audioData->channels; ++ch)
+                                        {
+                                                ma_uint64 frameIndex = i * audioData->channels + ch;
+                                                ma_int32 originalSample = frames[frameIndex];
+                                                double sample = (double)originalSample;
+
+                                                sample *= totalGainFactor;
+
+                                                // Clamp
+                                                if (sample > 2147483647.0)
+                                                        sample = 2147483647.0;
+                                                else if (sample < -2147483648.0)
+                                                        sample = -2147483648.0;
+
+                                                frames[frameIndex] = (ma_int32)sample;
+                                        }
+                                }
+                                break;
+
+                        default:
+                                break;
+                        }
+                }
+
                 if (((audioData->totalFrames != 0 && cursor != 0 && cursor >= audioData->totalFrames) || framesToRead == 0 || isSkipToNext() || result != MA_SUCCESS) && !isEOFReached())
                 {
                         activateSwitch(audioData);
@@ -193,18 +293,7 @@ void builtin_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_u
                 pthread_mutex_unlock(&dataSourceMutex);
         }
 
-        ma_int32 *audioBuffer = getAudioBuffer();
-        if (audioBuffer == NULL)
-        {
-                audioBuffer = malloc(sizeof(ma_int32) * MAX_BUFFER_SIZE);
-                if (audioBuffer == NULL)
-                {
-                        return;
-                }
-        }
-
-        memcpy(audioBuffer, pFramesOut, sizeof(ma_int32) * framesRead);
-        setAudioBuffer(audioBuffer);
+        setAudioBuffer(pFramesOut, framesRead);
 
         if (pFramesRead != NULL)
         {
@@ -216,6 +305,6 @@ void builtin_on_audio_frames(ma_device *pDevice, void *pFramesOut, const void *p
 {
         AudioData *pDataSource = (AudioData *)pDevice->pUserData;
         ma_uint64 framesRead = 0;
-        builtin_read_pcm_frames(&pDataSource->base, pFramesOut, frameCount, &framesRead);
+        builtin_read_pcm_frames(&(pDataSource->base), pFramesOut, frameCount, &framesRead);
         (void)pFramesIn;
 }
