@@ -16,6 +16,7 @@ soundcommon.c
 #endif
 
 bool repeatEnabled = false;
+bool repeatListEnabled = false;
 bool shuffleEnabled = false;
 bool skipToNext = false;
 bool seekRequested = false;
@@ -33,7 +34,9 @@ _Atomic bool readingFrames = false;
 pthread_mutex_t dataSourceMutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t switchMutex = PTHREAD_MUTEX_INITIALIZER;
 ma_device device = {0};
-ma_int32 *audioBuffer = NULL;
+static ma_int32 audioBuffer[MAX_BUFFER_SIZE];
+static int writeHead = 0;
+bool bufferReady = false;
 AudioData audioData;
 int bufSize;
 ma_event switchAudioImpl;
@@ -141,7 +144,7 @@ void setNextDecoder(void **decoderArray, void **decoder, void **firstDecoder, in
         {
                 *firstDecoder = *decoder;
         }
-        else if (*decoderIndex == -1) // array hasn't been used yet
+        else if (*decoderIndex == -1) // Array hasn't been used yet
         {
                 if (decoderArray[0] != NULL)
                 {
@@ -199,10 +202,10 @@ ma_decoder *getCurrentBuiltinDecoder(void)
 
 void switchDecoder(int *decoderIndex)
 {
-    if (*decoderIndex == -1)
-        *decoderIndex = 0;
-    else
-        *decoderIndex = 1 - *decoderIndex;
+        if (*decoderIndex == -1)
+                *decoderIndex = 0;
+        else
+                *decoderIndex = 1 - *decoderIndex;
 }
 
 #ifdef USE_FAAD
@@ -219,13 +222,14 @@ m4a_decoder *getCurrentM4aDecoder(void)
                 return m4aDecoders[m4aDecoderIndex];
 }
 
-void getM4aFileInfo(const char *filename, ma_format *format, ma_uint32 *channels, ma_uint32 *sampleRate, ma_channel *channelMap, k_m4adec_filetype *fileType)
+void getM4aFileInfo(const char *filename, ma_format *format, ma_uint32 *channels, ma_uint32 *sampleRate, ma_channel *channelMap, int *avgBitRate, k_m4adec_filetype *fileType)
 {
         m4a_decoder decoder;
         if (m4a_decoder_init_file(filename, NULL, NULL, &decoder) == MA_SUCCESS)
         {
                 *format = decoder.format;
                 m4a_decoder_get_data_format(&decoder, format, channels, sampleRate, channelMap, MA_MAX_CHANNELS);
+                *avgBitRate = decoder.avgBitRate / 1000;
                 *fileType = decoder.fileType;
                 m4a_decoder_uninit(&decoder, NULL);
         }
@@ -313,7 +317,7 @@ int prepareNextM4aDecoder(SongData *songData)
         decoder->onTell = m4a_get_cursor_in_pcm_frames_wrapper;
         decoder->cursor = 0;
 
-        setNextDecoder((void **)m4aDecoders, (void**)&decoder, (void**)&firstM4aDecoder, &m4aDecoderIndex, (uninit_func)uninitM4aDecoder);
+        setNextDecoder((void **)m4aDecoders, (void **)&decoder, (void **)&firstM4aDecoder, &m4aDecoderIndex, (uninit_func)uninitM4aDecoder);
 
         if (songData != NULL)
         {
@@ -359,30 +363,36 @@ ma_libopus *getCurrentOpusDecoder(void)
                 return opusDecoders[opusDecoderIndex];
 }
 
-ma_format getCurrentFormat(void)
+void getCurrentFormatAndSampleRate(ma_format *format, ma_uint32 *sampleRate)
 {
-        ma_format format = ma_format_unknown;
+        *format = ma_format_unknown;
 
-        if (getCurrentImplementationType() == BUILTIN)
+        if (isRadioPlaying())
+        {
+                ma_uint32 channels;
+                ma_data_source_get_data_format(radioContext.decoder.pBackend, format, &channels, sampleRate, NULL, 0);
+                return;
+        }
+        else if (getCurrentImplementationType() == BUILTIN)
         {
                 ma_decoder *decoder = getCurrentBuiltinDecoder();
 
                 if (decoder != NULL)
-                        format = decoder->outputFormat;
+                        *format = decoder->outputFormat;
         }
         else if (getCurrentImplementationType() == OPUS)
         {
                 ma_libopus *decoder = getCurrentOpusDecoder();
 
                 if (decoder != NULL)
-                        format = decoder->format;
+                        *format = decoder->format;
         }
         else if (getCurrentImplementationType() == VORBIS)
         {
                 ma_libvorbis *decoder = getCurrentVorbisDecoder();
 
                 if (decoder != NULL)
-                        format = decoder->format;
+                        *format = decoder->format;
         }
         else if (getCurrentImplementationType() == M4A)
         {
@@ -390,11 +400,11 @@ ma_format getCurrentFormat(void)
                 m4a_decoder *decoder = getCurrentM4aDecoder();
 
                 if (decoder != NULL)
-                        format = decoder->format;
+                        *format = decoder->format;
 #endif
         }
 
-        return format;
+        *sampleRate = audioData.sampleRate;
 }
 
 void getFileInfo(const char *filename, ma_uint32 *sampleRate, ma_uint32 *channels, ma_format *format)
@@ -511,7 +521,7 @@ int prepareNextVorbisDecoder(char *filepath)
                                                       channels == nchannels &&
                                                       sampleRate == nsampleRate));
 
-       if (!sameFormat)
+        if (!sameFormat)
         {
                 ma_libvorbis_uninit(decoder, NULL);
                 free(decoder);
@@ -529,7 +539,7 @@ int prepareNextVorbisDecoder(char *filepath)
         decoder->onSeek = ma_libvorbis_seek_to_pcm_frame_wrapper;
         decoder->onTell = ma_libvorbis_get_cursor_in_pcm_frames_wrapper;
 
-        setNextDecoder((void **)vorbisDecoders, (void**)&decoder, (void**)&firstVorbisDecoder, &vorbisDecoderIndex, (uninit_func)uninitVorbisDecoder);
+        setNextDecoder((void **)vorbisDecoders, (void **)&decoder, (void **)&firstVorbisDecoder, &vorbisDecoderIndex, (uninit_func)uninitVorbisDecoder);
 
         if (currentDecoder != NULL && decoder != NULL)
         {
@@ -577,7 +587,7 @@ int prepareNextDecoder(char *filepath)
                 free(decoder);
                 return -1;
         }
-        setNextDecoder((void **)decoders, (void**)&decoder, (void**)&firstDecoder, &decoderIndex, (uninit_func)uninitMaDecoder);
+        setNextDecoder((void **)decoders, (void **)&decoder, (void **)&firstDecoder, &decoderIndex, (uninit_func)uninitMaDecoder);
 
         if (currentDecoder != NULL && decoder != NULL)
         {
@@ -641,7 +651,7 @@ int prepareNextOpusDecoder(char *filepath)
         decoder->onSeek = ma_libopus_seek_to_pcm_frame_wrapper;
         decoder->onTell = ma_libopus_get_cursor_in_pcm_frames_wrapper;
 
-        setNextDecoder((void **)opusDecoders, (void**)&decoder, (void**)&firstOpusDecoder, &opusDecoderIndex, (uninit_func)uninitOpusDecoder);
+        setNextDecoder((void **)opusDecoders, (void **)&decoder, (void **)&firstOpusDecoder, &opusDecoderIndex, (uninit_func)uninitOpusDecoder);
 
         if (currentDecoder != NULL && decoder != NULL)
         {
@@ -661,17 +671,48 @@ void setBufferSize(int value)
         bufSize = value;
 }
 
-void initAudioBuffer(void)
+void setAudioBuffer(ma_int32 *buf, int numSamples)
 {
-        if (audioBuffer == NULL)
-        {
-                audioBuffer = malloc(sizeof(ma_int32) * MAX_BUFFER_SIZE);
-                if (audioBuffer == NULL)
-                {
-                        // Memory allocation failed
-                        return;
-                }
+    int bufIndex = 0;
+
+    while (bufIndex < numSamples)
+    {
+        if (writeHead >= MAX_BUFFER_SIZE) {
+            break;
         }
+
+        int spaceLeft = FFT_SIZE - writeHead;
+        int samplesLeft = numSamples - bufIndex;
+        int samplesToCopy = (samplesLeft < spaceLeft) ? samplesLeft : spaceLeft;
+
+        // Clamp copy to avoid buffer overflow
+        int maxCopy = MAX_BUFFER_SIZE - writeHead;
+        if (samplesToCopy > maxCopy)
+            samplesToCopy = maxCopy;
+
+        memcpy(&audioBuffer[writeHead], &buf[bufIndex], sizeof(ma_int32) * samplesToCopy);
+
+        writeHead += samplesToCopy;
+        bufIndex  += samplesToCopy;
+
+        // As long as we have at least FFT_SIZE samples, process one window
+        while (writeHead >= FFT_SIZE)
+        {
+            bufferReady = true;
+
+            // Shift the buffer left by HOP_SIZE
+            memmove(audioBuffer, audioBuffer + HOP_SIZE, sizeof(ma_int32) * (FFT_SIZE - HOP_SIZE));
+
+            writeHead -= HOP_SIZE;
+        }
+    }
+}
+
+void resetAudioBuffer(void)
+{
+    memset(audioBuffer, 0, sizeof(ma_int32) * MAX_BUFFER_SIZE);
+    writeHead = 0;
+    bufferReady = false;
 }
 
 ma_int32 *getAudioBuffer(void)
@@ -679,25 +720,6 @@ ma_int32 *getAudioBuffer(void)
         return audioBuffer;
 }
 
-void setAudioBuffer(ma_int32 *buf)
-{
-        audioBuffer = buf;
-}
-
-void resetAudioBuffer(void)
-{
-        memset(audioBuffer, 0, sizeof(float) * MAX_BUFFER_SIZE);
-}
-
-void freeAudioBuffer(void)
-{
-        if (audioBuffer != NULL)
-        {
-                free(audioBuffer);
-                audioBuffer = NULL;
-        }
-}
-
 bool isRepeatEnabled(void)
 {
         return repeatEnabled;
@@ -708,6 +730,16 @@ void setRepeatEnabled(bool value)
         repeatEnabled = value;
 }
 
+bool isRepeatListEnabled(void)
+{
+        return repeatListEnabled;
+}
+
+void setRepeatListEnabled(bool value)
+{
+        repeatListEnabled = value;
+}
+
 bool isShuffleEnabled(void)
 {
         return shuffleEnabled;
@@ -808,7 +840,7 @@ void seekPercentage(float percent)
 
 void resumePlayback(void)
 {
-        // if this was unpaused with no song loaded
+        // If this was unpaused with no song loaded
         if (audioData.restart && !isRadioPlaying())
         {
                 audioData.endOfListReached = false;
@@ -861,12 +893,8 @@ void pausePlayback(void)
 
 void cleanupPlaybackDevice(void)
 {
-        ma_device_stop(&device);
-        while (ma_device_get_state(&device) != ma_device_state_stopped && ma_device_get_state(&device) != ma_device_state_uninitialized)
-        {
-                c_sleep(100);
-        }
         ma_device_uninit(&device);
+        memset(&device, 0, sizeof(device));
 }
 
 void clearCurrentTrack(void)
@@ -903,26 +931,6 @@ bool isStopped(void)
         return stopped;
 }
 
-pthread_mutex_t deviceMutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t deviceStopped = PTHREAD_COND_INITIALIZER;
-
-void resetDevice(void)
-{
-        pthread_mutex_lock(&deviceMutex);
-
-        if (ma_device_get_state(&device) == ma_device_state_started)
-                ma_device_stop(&device);
-
-        while (ma_device_get_state(&device) == ma_device_state_started)
-        {
-                pthread_cond_wait(&deviceStopped, &deviceMutex);
-        }
-
-        pthread_mutex_unlock(&deviceMutex);
-
-        ma_device_uninit(&device);
-}
-
 ma_device *getDevice(void)
 {
         return &device;
@@ -1022,10 +1030,10 @@ int getSystemVolumeMac(void)
                 if (fgets(buf, sizeof(buf), fp) != NULL)
                 {
                         buf[strcspn(buf, "\n")] = '\0';
-                        int tempVolume = extractPercentageMac(buf);
-                        if (tempVolume != -1)
+                        int tmpVolume = extractPercentageMac(buf);
+                        if (tmpVolume != -1)
                         {
-                                currentVolume = tempVolume;
+                                currentVolume = tmpVolume;
                         }
                 }
                 pclose(fp);
@@ -1053,10 +1061,10 @@ int getSystemVolume(void)
         {
                 while (fgets(buf, sizeof(buf), fp) != NULL)
                 {
-                        int tempVolume = extractPercentage(buf);
-                        if (tempVolume != -1)
+                        int tmpVolume = extractPercentage(buf);
+                        if (tmpVolume != -1)
                         {
-                                currentVolume = tempVolume;
+                                currentVolume = tmpVolume;
                                 break;
                         }
                 }
@@ -1074,10 +1082,10 @@ int getSystemVolume(void)
                 {
                         while (fgets(buf, sizeof(buf), fp) != NULL)
                         {
-                                int tempVolume = extractPercentage(buf);
-                                if (tempVolume != -1)
+                                int tmpVolume = extractPercentage(buf);
+                                if (tmpVolume != -1)
                                 {
-                                        currentVolume = tempVolume;
+                                        currentVolume = tmpVolume;
                                         break;
                                 }
                         }
@@ -1159,7 +1167,7 @@ void m4a_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_uint6
                 m4a_decoder *decoder = getCurrentM4aDecoder();
 
                 if (pAudioData->totalFrames == 0)
-                        ma_data_source_get_length_in_pcm_frames(decoder, &pAudioData->totalFrames);
+                        ma_data_source_get_length_in_pcm_frames(decoder, &(pAudioData->totalFrames));
 
                 // Check if seeking is requested
                 if (isSeekRequested())
@@ -1223,20 +1231,7 @@ void m4a_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_uint6
                 pthread_mutex_unlock(&dataSourceMutex);
         }
 
-        ma_int32 *audioBuffer = getAudioBuffer();
-        if (audioBuffer == NULL)
-        {
-                audioBuffer = malloc(sizeof(ma_int32) * MAX_BUFFER_SIZE);
-                if (audioBuffer == NULL)
-                {
-                        // Memory allocation failed
-                        return;
-                }
-        }
-
-        // No format conversion needed, just copy the audio samples
-        memcpy(audioBuffer, pFramesOut, sizeof(ma_int32) * framesRead);
-        setAudioBuffer(audioBuffer);
+        setAudioBuffer(pFramesOut, framesRead);
 
         if (pFramesRead != NULL)
         {
@@ -1248,7 +1243,7 @@ void m4a_on_audio_frames(ma_device *pDevice, void *pFramesOut, const void *pFram
 {
         AudioData *pDataSource = (AudioData *)pDevice->pUserData;
         ma_uint64 framesRead = 0;
-        m4a_read_pcm_frames(&pDataSource->base, pFramesOut, frameCount, &framesRead);
+        m4a_read_pcm_frames(&(pDataSource->base), pFramesOut, frameCount, &framesRead);
         (void)pFramesIn;
 }
 #endif
@@ -1287,7 +1282,7 @@ void opus_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_uint
                 ma_libopus *decoder = getCurrentOpusDecoder();
 
                 if (pAudioData->totalFrames == 0)
-                        ma_data_source_get_length_in_pcm_frames(decoder, &pAudioData->totalFrames);
+                        ma_data_source_get_length_in_pcm_frames(decoder, &(pAudioData->totalFrames));
 
                 // Check if seeking is requested
                 if (isSeekRequested())
@@ -1351,20 +1346,7 @@ void opus_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_uint
                 pthread_mutex_unlock(&dataSourceMutex);
         }
 
-        ma_int32 *audioBuffer = getAudioBuffer();
-        if (audioBuffer == NULL)
-        {
-                audioBuffer = malloc(sizeof(ma_int32) * MAX_BUFFER_SIZE);
-                if (audioBuffer == NULL)
-                {
-                        // Memory allocation failed
-                        return;
-                }
-        }
-
-        // No format conversion needed, just copy the audio samples
-        memcpy(audioBuffer, pFramesOut, sizeof(ma_int32) * framesRead);
-        setAudioBuffer(audioBuffer);
+        setAudioBuffer(pFramesOut, framesRead);
 
         if (pFramesRead != NULL)
         {
@@ -1376,7 +1358,7 @@ void opus_on_audio_frames(ma_device *pDevice, void *pFramesOut, const void *pFra
 {
         AudioData *pDataSource = (AudioData *)pDevice->pUserData;
         ma_uint64 framesRead = 0;
-        opus_read_pcm_frames(&pDataSource->base, pFramesOut, frameCount, &framesRead);
+        opus_read_pcm_frames(&(pDataSource->base), pFramesOut, frameCount, &framesRead);
         (void)pFramesIn;
 }
 
@@ -1408,7 +1390,7 @@ void vorbis_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_ui
                 ma_libvorbis *decoder = getCurrentVorbisDecoder();
 
                 if (pAudioData->totalFrames == 0)
-                        ma_data_source_get_length_in_pcm_frames(decoder, &pAudioData->totalFrames);
+                        ma_data_source_get_length_in_pcm_frames(decoder, &(pAudioData->totalFrames));
 
                 if ((getCurrentImplementationType() != VORBIS && !isSkipToNext()) || (decoder == NULL))
                 {
@@ -1479,20 +1461,7 @@ void vorbis_read_pcm_frames(ma_data_source *pDataSource, void *pFramesOut, ma_ui
                 pthread_mutex_unlock(&dataSourceMutex);
         }
 
-        ma_int32 *audioBuffer = getAudioBuffer();
-        if (audioBuffer == NULL)
-        {
-                audioBuffer = malloc(sizeof(ma_int32) * MAX_BUFFER_SIZE);
-                if (audioBuffer == NULL)
-                {
-                        // Memory allocation failed
-                        return;
-                }
-        }
-
-        // No format conversion needed, just copy the audio samples
-        memcpy(audioBuffer, pFramesOut, sizeof(ma_int32) * framesRead);
-        setAudioBuffer(audioBuffer);
+        setAudioBuffer(pFramesOut, framesRead);
 
         if (pFramesRead != NULL)
         {
@@ -1504,6 +1473,6 @@ void vorbis_on_audio_frames(ma_device *pDevice, void *pFramesOut, const void *pF
 {
         AudioData *pDataSource = (AudioData *)pDevice->pUserData;
         ma_uint64 framesRead = 0;
-        vorbis_read_pcm_frames(&pDataSource->base, pFramesOut, frameCount, &framesRead);
+        vorbis_read_pcm_frames(&(pDataSource->base), pFramesOut, frameCount, &framesRead);
         (void)pFramesIn;
 }
