@@ -14,7 +14,6 @@ radio.c
 #define NI_MAXHOST 1025
 #define WAIT_TIMEOUT_SECONDS 3
 #define MAX_RECONNECT_RETRIES 20
-#define BUFFER_SIZE 8192
 
 typedef struct
 {
@@ -34,6 +33,9 @@ bool hasUpdatedServerList = false;
 RadioSearchResult *currentlyPlayingRadioStation = NULL;
 RadioPlayerContext radioContext = {0};
 int reconnectCounter = 0;
+bool radioIsActive = false;
+
+pthread_mutex_t radioLifecycleMutex = PTHREAD_MUTEX_INITIALIZER;
 
 typedef struct
 {
@@ -276,6 +278,19 @@ Country countries[] = {
     {"Zambia", "ZM"},
     {"Zimbabwe", "ZW"}};
 
+typedef struct
+{
+        char *searchTerm;
+        void (*callback)(const char *, const char *, const char *, const char *, const int, const int);
+        bool *stopFlag;
+} SearchThreadArgs;
+
+pthread_mutex_t server_list_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+pthread_t currentThread;
+
+bool stopFlag = false;
+
 const char *getCountryCode(const char *country_name)
 {
         for (int i = 0; i < (int)sizeof(countries) / (int)sizeof(countries[0]); i++)
@@ -340,25 +355,19 @@ bool isControlChar(char c)
         return (c >= 0 && c <= 31) || c == 127;
 }
 
-// Function to sanitize the URL by removing control characters and checking for malicious patterns
 bool sanitizeUrl(const char *url)
 {
-        // Check for common unsafe schemes
         if (strstr(url, "javascript:") != NULL ||
             strstr(url, "data:") != NULL ||
             strstr(url, "file:") != NULL ||
             strstr(url, "vbscript:") != NULL)
         {
-                fprintf(stderr, "Unsafe URL pattern detected: %s\n", url);
                 return false;
         }
-
-        // Check for control characters
         for (const char *p = url; *p; p++)
         {
                 if (isControlChar(*p))
                 {
-                        fprintf(stderr, "URL contains control character: %s\n", url);
                         return false;
                 }
         }
@@ -449,37 +458,68 @@ Server *pickServer()
         return NULL;
 }
 
-int internetRadioSearch(const char *searchTerm, void (*callback)(const char *, const char *, const char *, const char *, const int, const int))
+bool IsActiveRadio()
 {
-        CURL *curl;
-        Memory res = {0};
-        Server *server;
+        return radioIsActive;
+}
 
+void *searchThreadFunction(void *arg)
+{
+        SearchThreadArgs *args = (SearchThreadArgs *)arg;
+        CURL *curl = NULL;
+        Memory res = {.memory = NULL, .size = 0};
+        Server *server = NULL;
+        char *encodedTerm = NULL;
+
+        pthread_mutex_lock(&server_list_mutex);
         if (!hasUpdatedServerList)
         {
                 if (updateServerList() != 0)
                 {
-                        return -1;
-                }
+                        pthread_mutex_unlock(&server_list_mutex);
+                        free(args->searchTerm);
+                        free(args);
+                        setErrorMessage("Radio database unavailable.");
 
+                        return NULL;
+                }
                 hasUpdatedServerList = true;
         }
+        pthread_mutex_unlock(&server_list_mutex);
 
-        while ((server = pickServer()))
+        while (true)
         {
-                char url[2070];
+                pthread_mutex_lock(&server_list_mutex);
+                server = pickServer();
+                pthread_mutex_unlock(&server_list_mutex);
 
-                curl = curl_easy_init();
-                char *encodedTerm = curl_easy_escape(curl, searchTerm, 0);
-                if (!encodedTerm)
+                if (!server)
                 {
-                        fprintf(stderr, "Could not URL-encode the term\n");
-                        curl_easy_cleanup(curl);
-                        return -1;
+                        break;
                 }
 
+                free(res.memory);
+                res.memory = NULL;
+                res.size = 0;
+
+                curl = curl_easy_init();
+                if (!curl)
+                {
+                        continue;
+                }
+
+                encodedTerm = curl_easy_escape(curl, args->searchTerm, 0);
+                if (!encodedTerm)
+                {
+                        curl_easy_cleanup(curl);
+                        curl = NULL;
+                        continue;
+                }
+
+                char url[2070];
                 snprintf(url, sizeof(url), "%s/json/stations/byname/%s", server->url, encodedTerm);
                 curl_free(encodedTerm);
+                encodedTerm = NULL;
 
                 curl_easy_setopt(curl, CURLOPT_URL, url);
                 curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
@@ -489,45 +529,134 @@ int internetRadioSearch(const char *searchTerm, void (*callback)(const char *, c
                 snprintf(userAgent, sizeof(userAgent), "kew/%s", VERSION);
                 curl_easy_setopt(curl, CURLOPT_USERAGENT, userAgent);
                 curl_easy_setopt(curl, CURLOPT_TIMEOUT, 10L);
+                curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
                 curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 1L);
 
                 CURLcode result = curl_easy_perform(curl);
                 curl_easy_cleanup(curl);
+                curl = NULL;
 
-                if (result == CURLE_OK)
+                if (result == CURLE_OK && res.memory != NULL)
                 {
                         char *ptr = res.memory;
                         int count = 0;
-                        while ((ptr = strchr(ptr, '{')) && count < MAX_STATIONS)
+
+                        while ((ptr = strchr(ptr, '{')) != NULL && count < MAX_STATIONS)
                         {
-                                char name[128], resolved[4096], country[64], codec[32], bitrate[16], votes[16];
-                                GETF("\"name\"", name);
-                                GETF("\"url_resolved\"", resolved);
+                                char name[128] = {0}, resolved[4096] = {0}, country[64] = {0}, codec[32] = {0}, bitrate_str[16] = {0}, votes_str[16] = {0};
+                                int br = 0, vo = 0;
+
+                                if (!GETF("\"name\"", name))
+                                {
+                                        ptr++;
+                                        continue;
+                                }
+                                if (!GETF("\"url_resolved\"", resolved))
+                                {
+                                        ptr++;
+                                        continue;
+                                }
                                 GETF("\"country\"", country);
                                 GETF("\"codec\"", codec);
-                                int br = GETF("\"bitrate\"", bitrate) ? atoi(bitrate) : 0;
-                                int vo = GETF("\"votes\"", votes) ? atoi(votes) : 0;
+                                if (GETF("\"bitrate\"", bitrate_str))
+                                        br = atoi(bitrate_str);
+                                if (GETF("\"votes\"", votes_str))
+                                        vo = atoi(votes_str);
 
                                 if (!isSafeURL(resolved))
                                 {
                                         ptr++;
                                         continue;
                                 }
-                                callback(name, resolved, getCountryCode(country), codec, br, vo);
+                                args->callback(name, resolved, getCountryCode(country), codec, br, vo);
                                 count++;
                                 ptr++;
+
+                                if (count % 10 == 0)
+                                {
+                                        refresh = true;
+                                        c_sleep(100);
+                                }
                         }
+
                         free(res.memory);
+                        res.memory = NULL;
+                        res.size = 0;
 
-                        return 0;
+                        free(encodedTerm);
+                        if (curl)
+                                curl_easy_cleanup(curl);
+
+                        free(args->searchTerm);
+                        free(args);
+
+                        return NULL;
                 }
+                else
+                {
 
-                server->isBroken = true;
-                free(res.memory);
-                res.memory = NULL;
-                res.size = 0;
+                        pthread_mutex_lock(&server_list_mutex);
+                        server->isBroken = true;
+                        pthread_mutex_unlock(&server_list_mutex);
+
+                        free(res.memory);
+                        res.memory = NULL;
+                        res.size = 0;
+                }
         }
 
+        return NULL;
+}
+
+void stopCurrentThread()
+{
+        if (currentThread)
+        {
+                stopFlag = true;
+                pthread_cancel(currentThread);
+                pthread_join(currentThread, NULL);
+                stopFlag = false;
+        }
+}
+
+int internetRadioSearch(const char *searchTerm, void (*callback)(const char *, const char *, const char *, const char *, const int, const int))
+{
+        pthread_t threadId;
+        SearchThreadArgs *args;
+
+        if (!searchTerm || !callback)
+        {
+                return -1;
+        }
+
+        stopCurrentThread();
+
+        args = malloc(sizeof(SearchThreadArgs));
+        if (!args)
+        {
+                return -1;
+        }
+
+        args->searchTerm = strdup(searchTerm);
+        if (!args->searchTerm)
+        {
+                free(args);
+                return -1;
+        }
+
+        args->callback = callback;
+        args->stopFlag = &stopFlag;
+
+        if (pthread_create(&threadId, NULL, searchThreadFunction, args) != 0)
+        {
+                free(args->searchTerm);
+                free(args);
+                return -1;
+        }
+
+        pthread_detach(threadId);
+        currentThread = threadId;
+
         return 0;
 }
 
@@ -544,11 +673,11 @@ static size_t curl_writefunc(void *ptr, size_t size, size_t nmemb, void *userdat
         size_t bytes = size * nmemb;
         size_t written = 0;
 
-        pthread_mutex_lock(&buf->mutex);
+        pthread_mutex_lock(&(buf->mutex));
 
         if (buf->eof)
         {
-                pthread_mutex_unlock(&buf->mutex);
+                pthread_mutex_unlock(&(buf->mutex));
                 return 0;
         }
 
@@ -571,8 +700,8 @@ static size_t curl_writefunc(void *ptr, size_t size, size_t nmemb, void *userdat
         buf->last_data_time = time(NULL);
 
         // Signal to the consumer that the data is ready
-        pthread_cond_signal(&buf->cond);
-        pthread_mutex_unlock(&buf->mutex);
+        pthread_cond_signal(&(buf->cond));
+        pthread_mutex_unlock(&(buf->mutex));
 
         return bytes;
 }
@@ -583,10 +712,26 @@ static ma_result decoder_read_callback(ma_decoder *decoder, void *pBufferOut, si
         size_t total_read = 0;
         unsigned char *out = (unsigned char *)pBufferOut;
 
-        pthread_mutex_lock(&buf->mutex);
+#ifndef __APPLE__
+        int rc = pthread_mutex_lock(&(buf->mutex));
+        if (rc == EOWNERDEAD)
+        {
+                // Thread died without unlocking
+                pthread_mutex_consistent(&(buf->mutex));
+        }
+#else
+        pthread_mutex_lock(&(buf->mutex));
+#endif
 
         while (total_read < bytesToRead)
         {
+                if ((buf->read_pos == buf->write_pos) && buf->eof)
+                {
+                        pthread_mutex_unlock(&(buf->mutex));
+                        *bytesRead = total_read;
+                        return (total_read > 0) ? MA_SUCCESS : MA_AT_END;
+                }
+
                 // Wait until data is available OR EOF is flagged.
                 while ((buf->read_pos == buf->write_pos) && !buf->eof)
                 {
@@ -596,7 +741,7 @@ static ma_result decoder_read_callback(ma_decoder *decoder, void *pBufferOut, si
                         ts.tv_sec += WAIT_TIMEOUT_SECONDS;
 
                         // Use timed wait
-                        int wait_result = pthread_cond_timedwait(&buf->cond, &buf->mutex, &ts);
+                        int wait_result = pthread_cond_timedwait(&(buf->cond), &(buf->mutex), &ts);
                         if (wait_result == ETIMEDOUT)
                         {
                                 // Check how long it's been since last data arrived
@@ -604,7 +749,7 @@ static ma_result decoder_read_callback(ma_decoder *decoder, void *pBufferOut, si
                                 if ((now - buf->last_data_time) >= WAIT_TIMEOUT_SECONDS)
                                 {
                                         // We haven't received data in too long, so signal error.
-                                        pthread_mutex_unlock(&buf->mutex);
+                                        pthread_mutex_unlock(&(buf->mutex));
                                         *bytesRead = total_read;
 
                                         buf->stale = true;
@@ -616,7 +761,7 @@ static ma_result decoder_read_callback(ma_decoder *decoder, void *pBufferOut, si
 
                 if ((buf->read_pos == buf->write_pos) && buf->eof)
                 {
-                        pthread_mutex_unlock(&buf->mutex);
+                        pthread_mutex_unlock(&(buf->mutex));
                         *bytesRead = total_read;
                         return (total_read > 0) ? MA_SUCCESS : MA_AT_END;
                 }
@@ -629,12 +774,12 @@ static ma_result decoder_read_callback(ma_decoder *decoder, void *pBufferOut, si
                                      ? chunk
                                      : (bytesToRead - total_read);
 
-                memcpy(out + total_read, &buf->buffer[buf->read_pos], to_copy);
+                memcpy(out + total_read, &(buf->buffer[buf->read_pos]), to_copy);
                 buf->read_pos = (buf->read_pos + to_copy) % STREAM_BUFFER_SIZE;
                 total_read += to_copy;
         }
 
-        pthread_mutex_unlock(&buf->mutex);
+        pthread_mutex_unlock(&(buf->mutex));
 
         *bytesRead = total_read;
         return MA_SUCCESS;
@@ -652,17 +797,24 @@ static void audio_data_callback(ma_device *device, void *output, const void *inp
 {
         (void)input;
         ma_decoder *decoder = (ma_decoder *)device->pUserData;
-        ma_uint64 frames_read = 0;
-        ma_decoder_read_pcm_frames(decoder, output, frameCount, &frames_read);
-        if (frames_read < frameCount)
+        ma_uint64 framesRead = 0;
+
+        ma_decoder_read_pcm_frames(decoder, output, frameCount, &framesRead);
+        if (framesRead < frameCount)
         {
-                memset((char *)output + frames_read * device->playback.channels * sizeof(float), 0, (frameCount - frames_read) * device->playback.channels * sizeof(float));
+                memset((char *)output + framesRead * device->playback.channels * sizeof(float), 0, (frameCount - framesRead) * device->playback.channels * sizeof(float));
         }
+
+        setBufferSize(framesRead);
+
+        radioIsActive = true;
+
+        setAudioBuffer(output, framesRead);
 }
 
 RadioSearchResult *copyRadioSearchResult(const RadioSearchResult *original)
 {
-        if (!original)
+        if (!original || strlen(original->name) <= 0)
                 return NULL;
 
         RadioSearchResult *copy = malloc(sizeof(RadioSearchResult));
@@ -699,14 +851,46 @@ void freeCurrentlyPlayingRadioStation(void)
         }
 }
 
-void stopRadio(void)
+void initRadioMutexes()
 {
-        pthread_mutex_lock(&radioContext.buf.mutex);
+#ifndef __APPLE__
+        pthread_mutexattr_t attr;
+        pthread_mutexattr_init(&attr);
+        pthread_mutexattr_setrobust(&attr, PTHREAD_MUTEX_ROBUST);
+        pthread_mutex_init(&(radioContext.buf.mutex), &attr);
+#else
+        pthread_mutex_init(&(radioContext.buf.mutex), NULL);
+#endif
+
+        pthread_cond_init(&(radioContext.buf.cond), NULL);
+}
+
+void destroyRadioMutexes()
+{
+        pthread_mutex_destroy(&(radioContext.buf.mutex));
+        pthread_cond_destroy(&(radioContext.buf.cond));
+        pthread_mutex_destroy(&radioLifecycleMutex);
+}
+
+int stopRadio(void)
+{
+#ifndef __APPLE__
+        int rc = pthread_mutex_lock(&(radioContext.buf.mutex));
+        if (rc == EOWNERDEAD)
+        {
+                // Thread died without unlocking
+                pthread_mutex_consistent(&(radioContext.buf.mutex));
+        }
+#else
+        pthread_mutex_lock(&(radioContext.buf.mutex));
+#endif
+
         radioContext.buf.eof = 1;
-        radioContext.buf.read_pos = radioContext.buf.write_pos = 0;
         radioContext.buf.stale = false;
-        pthread_cond_broadcast(&radioContext.buf.cond);
-        pthread_mutex_unlock(&radioContext.buf.mutex);
+        pthread_cond_broadcast(&(radioContext.buf.cond));
+        pthread_mutex_unlock(&(radioContext.buf.mutex));
+
+        cleanupPlaybackDevice();
 
         if (radioContext.curl_thread)
         {
@@ -714,11 +898,7 @@ void stopRadio(void)
                 radioContext.curl_thread = 0;
         }
 
-        resetDevice();
-
-        memset(&device, 0, sizeof(device));
-
-        ma_decoder_uninit(&radioContext.decoder);
+        ma_decoder_uninit(&(radioContext.decoder));
 
         if (radioContext.curl != NULL)
         {
@@ -726,14 +906,17 @@ void stopRadio(void)
                 radioContext.curl = NULL;
         }
 
-        pthread_mutex_destroy(&radioContext.buf.mutex);
-        pthread_cond_destroy(&radioContext.buf.cond);
+        radioContext.buf.read_pos = radioContext.buf.write_pos = 0;
 
-        memset(&radioContext.decoder, 0, sizeof(ma_decoder));
+        memset(&(radioContext.decoder), 0, sizeof(ma_decoder));
 
         radioIsPlaying = false;
 
         freeCurrentlyPlayingRadioStation();
+
+        radioIsActive = false;
+
+        return 0;
 }
 
 void reconnectRadioIfNeeded()
@@ -766,51 +949,60 @@ void *curl_perform_wrapper(void *arg)
 
 int playRadioStation(const RadioSearchResult *station)
 {
+        if (isRadioPlaying() && !radioIsActive) // Don't stop and start if things haven't really started
+                return -1;
+
+        if (station == NULL)
+        {
+                return -1;
+        }
+
+        pthread_mutex_lock(&radioLifecycleMutex);
+
         if (isRadioPlaying())
         {
                 RadioSearchResult *radio = getCurrentPlayingRadioStation();
 
                 // If it's not the same station, reset the reconnect counter
-                if (strcmp(radio->url_resolved, station->url_resolved) != 0)
+                if (radio && strcmp(radio->url_resolved, station->url_resolved) != 0)
                         reconnectCounter = 0;
         }
 
-        stopRadio();
-
-        if (station == NULL)
-                return -1;
-
-        if (!(radioContext.curl = curl_easy_init()))
+        if (stopRadio() < 0)
         {
-                fprintf(stderr, "Curl init failed\n");
+                pthread_mutex_unlock(&radioLifecycleMutex);
+                return -1;
+        }
+
+        if (!(radioContext.curl = curl_easy_init()))
+        {
+                pthread_mutex_unlock(&radioLifecycleMutex);
                 return -1;
         }
 
-        pthread_mutex_init(&radioContext.buf.mutex, NULL);
-        pthread_cond_init(&radioContext.buf.cond, NULL);
         radioContext.buf.eof = 0;
         radioContext.buf.stale = false;
 
         curl_easy_setopt(radioContext.curl, CURLOPT_URL, station->url_resolved);
         curl_easy_setopt(radioContext.curl, CURLOPT_WRITEFUNCTION, curl_writefunc);
-        curl_easy_setopt(radioContext.curl, CURLOPT_WRITEDATA, &radioContext.buf);
+        curl_easy_setopt(radioContext.curl, CURLOPT_WRITEDATA, &(radioContext.buf));
         curl_easy_setopt(radioContext.curl, CURLOPT_FOLLOWLOCATION, 1L);
 
         char userAgent[64];
         snprintf(userAgent, sizeof(userAgent), "kew/%s", VERSION);
         curl_easy_setopt(radioContext.curl, CURLOPT_USERAGENT, userAgent);
 
-        pthread_create(&radioContext.curl_thread, NULL, curl_perform_wrapper, radioContext.curl);
+        pthread_create(&(radioContext.curl_thread), NULL, curl_perform_wrapper, radioContext.curl);
 
         ma_decoder_config decoderConfig = ma_decoder_config_init(ma_format_f32, 2, 44100);
 
         decoderConfig.encodingFormat = ma_encoding_format_mp3;
 
-        if (ma_decoder_init(decoder_read_callback, decoder_seek_callback, &radioContext.buf, &decoderConfig, &radioContext.decoder) != MA_SUCCESS)
+        if (ma_decoder_init(decoder_read_callback, decoder_seek_callback, &(radioContext.buf), &decoderConfig, &(radioContext.decoder)) != MA_SUCCESS)
         {
                 fprintf(stderr, "Decoder init failed\n");
                 setErrorMessage("Radio station unsupported or unavailable.");
-
+                pthread_mutex_unlock(&radioLifecycleMutex);
                 return -1;
         }
 
@@ -819,12 +1011,13 @@ int playRadioStation(const RadioSearchResult *station)
         devConfig.playback.channels = radioContext.decoder.outputChannels;
         devConfig.sampleRate = radioContext.decoder.outputSampleRate;
         devConfig.dataCallback = audio_data_callback;
-        devConfig.pUserData = &radioContext.decoder;
+        devConfig.pUserData = &(radioContext.decoder);
 
         if (ma_device_init(NULL, &devConfig, &device) != MA_SUCCESS)
         {
                 fprintf(stderr, "Device init failed\n");
-                ma_decoder_uninit(&radioContext.decoder);
+                ma_decoder_uninit(&(radioContext.decoder));
+                pthread_mutex_unlock(&radioLifecycleMutex);
                 return -1;
         }
 
@@ -834,20 +1027,28 @@ int playRadioStation(const RadioSearchResult *station)
         {
                 fprintf(stderr, "Failed to start device playback\n");
                 ma_device_uninit(&device);
-                ma_decoder_uninit(&radioContext.decoder);
+                ma_decoder_uninit(&(radioContext.decoder));
+                pthread_mutex_unlock(&radioLifecycleMutex);
                 return -1;
         }
 
+        while (ma_device_get_state(&device) == ma_device_state_starting)
+        {
+                c_sleep(100);
+        }
+
         stopped = false;
         paused = false;
 
         refresh = true;
 
-        pthread_cond_signal(&radioContext.buf.cond);
+        pthread_cond_signal(&(radioContext.buf.cond));
 
         radioIsPlaying = true;
 
         setCurrentlyPlayingRadioStation(station);
 
+        pthread_mutex_unlock(&radioLifecycleMutex);
+
         return 0;
 }
