@@ -1,6 +1,8 @@
 #ifndef SOUND_RADIO_H
 #define SOUND_RADIO_H
 
+#include <pthread.h>
+#include <stdatomic.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,6 +46,7 @@ typedef struct
         CURL *curl;
         pthread_t curl_thread;
         ma_decoder decoder;
+        bool isRunning;
 } RadioPlayerContext;
 
 extern RadioPlayerContext radioContext;
@@ -52,16 +55,22 @@ int internetRadioSearch(const char *searchTerm, void (*callback)(const char *, c
 
 int playRadioStation(const RadioSearchResult *station);
 
-void stopRadio(void);
+int stopRadio(void);
 
 void reconnectRadioIfNeeded();
 
 bool isRadioPlaying(void);
 
+bool IsActiveRadio(void);
+
 RadioSearchResult *getCurrentPlayingRadioStation(void);
 
 void setCurrentlyPlayingRadioStation(const RadioSearchResult * result);
 
 void freeCurrentlyPlayingRadioStation(void);
 
+void initRadioMutexes(void);
+
+void destroyRadioMutexes(void);
+
 #endif
