@@ -23,6 +23,10 @@
 #include <taglib/wavfile.h>
 #include <taglib/xiphcomment.h>
 #include <taglib/tag.h>
+#include <taglib/apefooter.h>
+#include <taglib/apeitem.h>
+#include <taglib/apetag.h>
+#include <taglib/textidentificationframe.h>
 #include <ogg/ogg.h>
 
 /*
@@ -319,7 +323,7 @@ extern "C"
                                         // Clean up stream states
                                         for (auto &streamEntry : streams)
                                         {
-                                                ogg_stream_clear(&streamEntry.second);
+                                                ogg_stream_clear(&(streamEntry.second));
                                         }
                                         return false; // Could not write to output file
                                 }
@@ -334,7 +338,7 @@ extern "C"
                 // Clean up stream states
                 for (auto &streamEntry : streams)
                 {
-                        ogg_stream_clear(&streamEntry.second);
+                        ogg_stream_clear(&(streamEntry.second));
                 }
 
                 // Return whether the cover art was successfully found and written
@@ -698,7 +702,7 @@ extern "C"
 
                         // Copy the result into the output char* title, ensuring no overflow
                         c_strcpy(title, extractedTitle.c_str(), titleMaxLength - 1); // Copy up to titleMaxLength - 1 characters
-                        title[titleMaxLength - 1] = '\0';                           // Null-terminate the string
+                        title[titleMaxLength - 1] = '\0';                            // Null-terminate the string
                 }
                 else
                 {
@@ -707,10 +711,35 @@ extern "C"
                 }
         }
 
+        double parseDecibelValue(const TagLib::String &dbString)
+        {
+                double val = 0.0;
+                try
+                {
+                        std::string valStr = dbString.to8Bit(true);
+                        std::string filtered;
+                        for (char c : valStr)
+                        {
+                                if (std::isdigit((unsigned char)c) || c == '.' || c == '-' || c == '+' || c == 'e' || c == 'E')
+                                {
+                                        filtered.push_back(c);
+                                }
+                        }
+                        val = std::stod(filtered);
+                }
+                catch (...)
+                {
+                }
+                return val;
+        }
+
         int extractTags(const char *input_file, TagSettings *tag_settings, double *duration, const char *coverFilePath)
         {
                 memset(tag_settings, 0, sizeof(TagSettings)); // Initialize tag settings
 
+                tag_settings->replaygainTrack = 0.0;
+                tag_settings->replaygainAlbum = 0.0;
+
                 // Use TagLib's FileRef for generic file parsing.
                 TagLib::FileRef f(input_file);
                 if (f.isNull() || !f.file())
@@ -776,7 +805,95 @@ extern "C"
                         return -2;
                 }
 
-                // Cover art extraction using format-specific classes if necessary.
+                // Extract replay gain information
+                if (std::string(input_file).find(".mp3") != std::string::npos)
+                {
+                        TagLib::MPEG::File mp3File(input_file);
+                        TagLib::ID3v2::Tag *id3v2Tag = mp3File.ID3v2Tag();
+
+                        if (id3v2Tag)
+                        {
+                                // Retrieve all TXXX frames
+                                TagLib::ID3v2::FrameList frames = id3v2Tag->frameList("TXXX");
+                                for (TagLib::ID3v2::FrameList::Iterator it = frames.begin();
+                                     it != frames.end(); ++it)
+                                {
+                                        // Cast to the user-text (TXXX) frame class
+                                        TagLib::ID3v2::TextIdentificationFrame *txxx =
+                                            dynamic_cast<TagLib::ID3v2::TextIdentificationFrame *>(*it);
+                                        if (!txxx)
+                                                continue;
+
+                                        TagLib::StringList fields = txxx->fieldList();
+                                        if (fields.size() >= 2)
+                                        {
+                                                TagLib::String desc = fields[0];
+                                                TagLib::String val = fields[1];
+
+                                                if (desc.upper() == "REPLAYGAIN_TRACK_GAIN")
+                                                {
+                                                        tag_settings->replaygainTrack = parseDecibelValue(val);
+                                                }
+                                                else if (desc.upper() == "REPLAYGAIN_ALBUM_GAIN")
+                                                {
+                                                        tag_settings->replaygainAlbum = parseDecibelValue(val);
+                                                }
+                                        }
+                                }
+                        }
+
+                        TagLib::APE::Tag *apeTag = mp3File.APETag();
+
+                        if (apeTag)
+                        {
+                                TagLib::APE::ItemListMap items = apeTag->itemListMap();
+                                for (auto it = items.begin(); it != items.end(); ++it)
+                                {
+                                        std::string key = it->first.upper().toCString();
+                                        TagLib::String value = it->second.toString();
+
+                                        if (key == "REPLAYGAIN_TRACK_GAIN")
+                                        {
+                                                tag_settings->replaygainTrack = parseDecibelValue(value);
+                                        }
+                                        else if (key == "REPLAYGAIN_ALBUM_GAIN")
+                                        {
+                                                tag_settings->replaygainAlbum = parseDecibelValue(value);
+                                        }
+                                }
+                        }
+                }
+                else if (std::string(input_file).find(".flac") != std::string::npos)
+                {
+                        TagLib::FLAC::File flacFile(input_file);
+                        TagLib::Ogg::XiphComment *xiphComment = flacFile.xiphComment();
+
+                        if (xiphComment)
+                        {
+                                const TagLib::Ogg::FieldListMap &fieldMap = xiphComment->fieldListMap();
+
+                                auto trackGainIt = fieldMap.find("REPLAYGAIN_TRACK_GAIN");
+                                if (trackGainIt != fieldMap.end())
+                                {
+                                        const TagLib::StringList &trackGainList = trackGainIt->second;
+                                        if (!trackGainList.isEmpty())
+                                        {
+                                                tag_settings->replaygainTrack = parseDecibelValue(trackGainList.front());
+                                        }
+                                }
+
+                                auto albumGainIt = fieldMap.find("REPLAYGAIN_ALBUM_GAIN");
+                                if (albumGainIt != fieldMap.end())
+                                {
+                                        const TagLib::StringList &albumGainList = albumGainIt->second;
+                                        if (!albumGainList.isEmpty())
+                                        {
+                                                tag_settings->replaygainAlbum = parseDecibelValue(albumGainList.front());
+                                        }
+                                }
+                        }
+                }
+
                 std::string filename(input_file);
                 std::string extension = filename.substr(filename.find_last_of('.') + 1);
                 bool coverArtExtracted = false;
