@@ -10,6 +10,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <glib.h>
+#include <math.h>
 #include <pwd.h>
 #include <regex.h>
 #include <stdbool.h>
@@ -62,10 +63,10 @@ void shortenString(char *str, size_t maxLength);
 
 void printBlankSpaces(int numSpaces);
 
-int compareLibEntries(const struct dirent **a, const struct dirent **b);
-
-int compareLibEntriesReversed(const struct dirent **a, const struct dirent **b);
-
 int getNumber(const char *str);
 
+char *getFilePath(const char *filename);
+
+float getFloat(const char *str);
+
 #endif
