@@ -3,14 +3,12 @@
 #include <float.h>
 #include <fftw3.h>
 #include <math.h>
-#include <locale.h>
 #include <complex.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include "sound.h"
 #include "term.h"
 #include "utils.h"
-#include <wchar.h>
 
 #ifndef PIXELDATA_STRUCT
 #define PIXELDATA_STRUCT
@@ -22,10 +20,10 @@
     } PixelData;
 #endif
 
-void initVisuals();
+void initVisuals(void);
 
-void freeVisuals();
+void freeVisuals(void);
 
-void drawSpectrumVisualizer(int height, int width, PixelData c, int indentation, bool useProfileColors);
+void drawSpectrumVisualizer(AppState *state, int indentation);
 
 PixelData increaseLuminosity(PixelData pixel, int amount);
